# Created for BADS 2018
# See README.md for details
# Python 3
"""
This module implements the topological order algorithm described in 
Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne. For more
information, see chapter 4.2 of the book.
"""

from itu.algs4.graphs.depth_first_order import DepthFirstOrder
from itu.algs4.graphs.digraph import Digraph
from itu.algs4.graphs.directed_cycle import DirectedCycle
from itu.algs4.graphs.edge_weighted_directed_cycle import EdgeWeightedDirectedCycle


class Topological:
    """
    The Topological class represents a data type for determining a topological
    order of a directed acyclic graph (DAG). Recall, a digraph has a topological 
    order if and only if it is a DAG. The hasOrder operation determines whether 
    the digraph has a topological order, and if so, the order operation returns one.
    
    This implementation uses depth-first search. The constructor takes time 
    proportional to V + E (in the worst case), where V is the number of vertices 
    and E is the number of edges. Afterwards, the hasOrder and rank operations 
    takes constant time the order operation takes time proportional to V.

    See DirectedCycle, DirectedCycleX, and EdgeWeightedDirectedCycle to compute 
    a directed cycle if the digraph is not a DAG. See TopologicalX for a 
    nonrecursive queue-based algorithm to compute a topological order of a DAG.

    For additional documentation, see Section 4.2 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """
    
    def __init__(self, digraph):
        """
        Determines whether the digraph (or edge weighted digraph) 
        has a topological order and, if so, finds such a topological order.
        
        :param digraph: the Digraph or EdgeWeightedDigraph to check
        """
        self._order = None
        
        if isinstance(digraph, Digraph):
            finder = DirectedCycle(digraph)
        else:
            finder = EdgeWeightedDirectedCycle(digraph)

        if not finder.has_cycle():
            dfs = DepthFirstOrder(digraph)
            self._order = dfs.reverse_post()
            self._rank = [0]*digraph.V()
            i = 0
            for v in self._order:
                self._rank[v] = i
                i += 1
    
    def order(self):
        """
        Returns a topological order if the digraph has a topologial order, and None otherwise.
        
        :returns: a topological order of the vertices (as an interable) if the digraph has a 
                 topological order (or equivalently, if the digraph is a DAG), and None otherwise
        """
        return self._order
    
    def has_order(self):
        """
        Does the digraph have a topological order?
        
        :returns: True if the digraph has a topological order (or equivalently, if the digraph 
                 is a DAG), and False otherwise
        """
        return self._order != None
    
    def rank(self, v):
        """
        The the rank of vertex v in the topological order -1 if the digraph is not a DAG
        
        :param v: the vertex
        :returns: the position of vertex v in a topological order of the digraph -1 if the digraph is not a DAG
        """
        self._validate_vertex(v)
        if self.has_order():
            return self._rank[v]
        else:
            return -1
    
    def _validate_vertex(self, v):
        # throw an IllegalArgumentException unless 0 <= v < V        
        V = len(self._rank)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}", v, (V-1))


if __name__ == '__main__':
    import sys
    from itu.algs4.stdio.instream import InStream
    from itu.algs4.stdlib import stdio
    from itu.algs4.graphs.symbol_digraph import SymbolDigraph

    filename  = sys.argv[1]
    delimiter = sys.argv[2]
    sg = SymbolDigraph(filename, delimiter)
    topological = Topological(sg.digraph())
    for v in topological.order():
        stdio.writeln(sg.name_of(v))



# Created for BADS 2018
# See README.md for details
# Python 3
import sys

from itu.algs4.stdlib import stdio

#  Execution:    python frequency_counter.py L < input.txt
#
#  Read in a list of words from standard input and print out
#  the most frequently occurring word that has length greater than
#  a given threshold.
#
#  % python frequency_counter.py 1 < tinyTale.txt
#  it 10
#
#  % python frequency_counter.py 8 < tale.txt
#  business 122
#
#  % python frequency_counter.py 10 < leipzig1M.txt
#  government 24763


""" 
  The FrequencyCounter class provides a client for 
  reading in a sequence of words and printing a word (exceeding
  a given length) that occurs most frequently. It is useful as
  a test client for various symbol table implementations.

  Reads in a command-line integer and sequence of words from
  standard input and prints out a word (whose length exceeds
  the threshold) that occurs most frequently to standard output.
  It also prints out the number of words whose length exceeds
  the threshold and the number of distinct such words.
"""

if __name__ == '__main__':
    args = sys.argv[1:]
    distinct = 0
    words = 0
    minlen = int(args[0])
    st = {}

    #compute frequency counts
    while not stdio.isEmpty():
        key = stdio.readString()
        if len(key) < minlen: 
            continue
        words+=1
        if key in st:                   
           st[key] = st.get(key) + 1    
        else:
            st[key] = 1                 
            distinct+=1

    # find a key with the highest frequency count
    max = ""
    st[max] = 0                         
    for word in st.keys():
        if st.get(word) > st.get(max):
            max = word


    print(max + " " + str(st.get(max)))
    print("distinct = " + str(distinct))
    print("words    = " + str(words))



# Created for BADS 2018
# see README.md for details
# This is python3 

from itu.algs4.fundamentals.bag import Bag
from itu.algs4.fundamentals.stack import Stack


class Graph:
    """
    The Graph class represents an undirected graph of vertices
    named 0 through V - 1.
    It supports the following two primary operations: add an edge to the graph,
    iterate over all of the vertices adjacent to a vertex. It also provides
    methods for returning the number of vertices V and the number
    of edges E. Parallel edges and self-loops are permitted.
    By convention, a self-loop v-v appears in the
    adjacency list of v twice and contributes two to the degree
    of v.

    This implementation uses an adjacency-lists representation, which 
    is a vertex-indexed array of Bag objects.
    All operations take constant time (in the worst case) except
    iterating over the vertices adjacent to a given vertex, which takes
    time proportional to the number of such vertices.
    """
    def __init__(self, V):
        """
        Initializes an empty graph with V vertices and 0 edges.
        param V the number of vertices

        :param V: number of vertices
        :raises: ValueError if V < 0
        """
        if V < 0: raise ValueError("Number of vertices must be nonnegative")
        self._V = V     # number of vertices
        self._E = 0     # number of edges
        self._adj = []  # adjacency lists

        for _ in range(V):
            self._adj.append(Bag()) # Initialize all lists to empty bags.

    @staticmethod
    def from_stream(stream):
        """
        Initializes a graph from the specified input stream.
        The format is the number of vertices V,
        followed by the number of edges E,
        followed by E pairs of vertices, with each entry separated by whitespace.
      
        :param stream: the input stream
        :returns: new graph from stream
        :raises ValueError: if the endpoints of any edge are not in prescribed range
        :raises ValueError: if the number of vertices or edges is negative
        :raises ValueError: if the input stream is in the wrong format 
        """
        V = stream.readInt()        # read V
        if V < 0: raise ValueError("Number of vertices must be nonnegative")
        g = Graph(V)                # construct this graph
        E = stream.readInt()        # read E    
        if E < 0: raise ValueError("Number of edges in a Graph must be nonnegative")
        for _ in range(E):
            # Add an edge
            v = stream.readInt()    # read a vertex,
            w = stream.readInt()        # read another vertex,
            g._validateVertex(v)
            g._validateVertex(w)
            g.add_edge(v, w)             # and add edge connecting them.
        return g

    @staticmethod
    def from_graph(G):
        """
        Initializes a new graph that is a deep copy of G

        :param G: the graph to copy
        :returns: copy of G 
        """
        g = Graph(G.V())
        g._E = G.E()
        for v in range(G.V()):
            # reverse so that adjacency list is in same order as original
            reverse = Stack()
            for w in G._adj[v]:
                reverse.push(w)
            for w in reverse:
                g._adj[v].add(w)

    def V(self):
        """
        Returns the number of vertices in this graph.

        :returns: the number of vertices in this graph.
        """
        return self._V

    def E(self):
        """
        Returns the number of edges in this graph.

        :returns: the number of edges in this graph.
        """
        return self._E

    def _validateVertex(self, v):
        # throw a ValueError unless 0 <= v < V
        if v < 0 or v >= self._V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, self._V))

    def add_edge(self, v, w):
        """
        Adds the undirected edge v-w to this graph.

        :param v: one vertex in the edge
        :param w: the other vertex in the edge
        :raises ValueError: unless both 0 <= v < V and 0 <= w < V
        """
        self._adj[v].add(w)         # add w to v's list
        self._adj[w].add(v)         # add v to w's list
        self._E += 1

    def adj(self, v):
        """
        Returns the vertices adjacent to vertex v.
     
        :param v: the vertex
        :returns: the vertices adjacent to vertex v, as an iterable
        :raises ValueError: unless  0 <= v < V
        """
        self._validateVertex(v)        
        return self._adj[v]

    def degree(self, v):
        """
        Returns the degree of vertex v.
     
        :param v: the vertex
        :returns: the degree of vertex v
        :raises ValueError:  unless 0 <= v < V
        """
        self._validateVertex(v)
        return self._adj[v].size()

    def __repr__(self):
        """
        Returns a string representation of this graph.
     
        :returns: the number of vertices V, followed by the number of edges E,
                    followed by the V adjacency lists
        """
        s = ["{} vertices, {} edges\n".format(self._V, self._E)]
        for v in range(self._V):
            s.append("%d : " % (v))
            for w in self._adj[v]:
                s.append("%d " % (w))
            s.append("\n")

        return ''.join(s)

if __name__ == "__main__":
    from itu.algs4.stdlib.instream import InStream
    from itu.algs4.stdlib import stdio
    import sys

    In = InStream(sys.argv[1])
    G = Graph.from_stream(In)
    stdio.writeln(G)



# Created for BADS 2018
# See README.md for details
# Python 3

import sys
from typing import List, Optional

from itu.algs4.stdlib import stdio

"""
This module provides functions for sorting an array using mergesort.

For additional documentation, see Section 2.2 of Algorithms, 4th Edition 
by Robert Sedgewick and Kevin Wayne.
"""
#  Sorts a sequence of strings from standard input using mergesort

def _is_sorted(a: List, lo: int = 0, hi: Optional[int] = None):
    # If hi is not specified, use whole array
    if hi is None:
        hi = len(a)
        
    # check if sublist is sorted
    for i in range(lo+1, hi):
        if a[i] < a[i-1]:
            return False
    return True

# stably merge a[lo .. mid] with a[mid+1 ..hi] using aux[lo .. hi]
def _merge(a: List, aux: List, lo: int, mid: int, hi: int):
    # precondition: a[lo .. mid] and a[mid+1 .. hi] are sorted subarrays
    assert _is_sorted(a, lo, mid)
    assert _is_sorted(a, mid+1, hi)
    
    # copy to aux[]
    for k in range(lo, hi+1):
        aux[k] = a[k]
        
    # merge back to a[]
    i, j = lo, mid+1
    for k in range(lo, hi+1):
        if i > mid:
            a[k] = aux[j]
            j += 1
        elif j > hi:
            a[k] = aux[i]
            i += 1
        elif aux[j] < aux[i]:
            a[k] = aux[j]
            j += 1
        else:
            a[k] = aux[i]
            i += 1
    
    # postcondition: a[lo .. hi] is sorted
    assert _is_sorted(a, lo, hi)

    # mergesort a[lo..hi] using auxiliary array aux[lo..hi]
def _sort(a: List, aux: List, lo: int, hi: int):
    if (hi <= lo):
        return
    mid = lo + (hi-lo)//2
    _sort(a, aux, lo, mid)
    _sort(a, aux, mid+1, hi)
    _merge(a, aux, lo, mid, hi)

def sort(a: List):
    """
    Rearranges the array in ascending order, using the natural order.
    
    :param a: the array to be sorted
    """
    aux = [None]*len(a)
    _sort(a, aux, 0, len(a)-1)
    assert _is_sorted(a)
    


# Reads in a sequence of strings from standard input or a file 
# supplied as argument to the program; mergesorts them; 
# and prints them to standard output in ascending order.
if __name__ == '__main__':
    if len(sys.argv) > 1:
        try: 
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")
    
    a = stdio.readAllStrings()
    sort(a)
    assert _is_sorted(a)
    for elem in a:
        print(elem)



# Created for BADS 2018
# See README.md for details
# This is python3 
"""
The Huffman compression module provides static methods for compressing
and expanding a binary input using Huffman codes over the 8-bit extended
ASCII alphabet

For additional documentation, see Section 5.5 of Algorithms, 4th edtition
by Robert Sedgewick and Kevin Wayne.
"""
import sys

from itu.algs4.sorting.min_pq import MinPQ
from itu.algs4.stdlib.binary_stdin import BinaryStdIn
from itu.algs4.stdlib.binary_stdout import BinaryStdOut

_R = 256
class _Node:
	def __init__(self, ch, freq, left, right):
		self.ch = ch
		self.freq = freq
		self.left = left
		self.right = right
	def is_leaf(self):
		left = self.left
		right = self.right
		assert(left is None and right is None or left is not None and right is not None)
		return left is None and right is None
	def __gt__(self, that):
		return self.freq > that.freq
def compress():
	"""
	Reads a sequence of 8-bit bytes from standard input; compresses them
	using Huffman codes with an 8-bit alphabet; and writes the results
	to standard input.
	"""
	s = BinaryStdIn.read_string()
	#Tabulate frequency counts
	freq = [0 for i in range(0,_R)]
	for i in range(0,len(s)):
		freq[ord(s[i])] += 1
	#Build Huffman trie
	root = _build_trie(freq)
	#Build code table
	st = [None for i in range(0,_R)]
	_build_code(st, root, "")
	#Print trie for decoder
	_write_trie(root)
	#Print number of bytes in original uncompressed message
	BinaryStdOut.write_int(len(s))
	#Use Huffman code to encode input
	for i in range (0,len(s)):
		code = st[ord(s[i])]
		for j in range(0,len(code)):
			if(code[j]=='0'):
				BinaryStdOut.write_bool(False)
			elif(code[j]=='1'):
				BinaryStdOut.write_bool(True)
			else:
				raise ValueError("Illegal state")
	BinaryStdOut.close()
	
#Build the Huffman trie given frequencies
def _build_trie(freq):
	pq = MinPQ()
	for i in range(0,_R):
		if(freq[i] > 0):
			pq.insert(_Node(chr(i), freq[i], None, None))
	if(pq.size() == 0):
		raise ValueError("The provided file is empty")
	if(pq.size() == 1):
		if(freq[ord('\0')]==0):
			pq.insert(_Node('\0', 0, None, None))
		else:
			pq.insert(_Node('\1', 0, None, None))
	while(pq.size() > 1):
		left = pq.del_min()
		right = pq.del_min()
		parent = _Node('\0', left.freq + right.freq, left, right)
		pq.insert(parent)
	return pq.del_min()

#Write bitstring-encoded trie to standard output
def _write_trie(x):
	if(x.is_leaf()):
		BinaryStdOut.write_bool(True)
		BinaryStdOut.write_char(x.ch)
		return
	BinaryStdOut.write_bool(False)
	_write_trie(x.left)
	_write_trie(x.right)
	
#Make a lookup table from symbols and their encodings
def _build_code(st, x, s):
	if(not x.is_leaf()):
		_build_code(st, x.left, s+'0')
		_build_code(st, x.right, s+'1')
	else:
		st[ord(x.ch)] = s
		

def expand():
	"""
	Reads a sequence of bits that represents a Huffman-compressed message from
	standard input; expands them; and writes the results to standard output.
	"""
	BinaryStdIn.is_empty()
	root = _read_trie()
	length = BinaryStdIn.read_int()
	for i in range(0,length):
		x = root
		while(not x.is_leaf()):
			bit = BinaryStdIn.read_bool()
			if(bit):
				x = x.right
			else:
				x = x.left
		BinaryStdOut.write_char(x.ch)
	BinaryStdOut.close()
	
def _read_trie():
	isLeaf = BinaryStdIn.read_bool()
	if(isLeaf):
		return _Node(BinaryStdIn.read_char(), -1, None, None)
	else:
		return _Node('\0', -1, _read_trie(), _read_trie())
	
def main():
	"""
	Sample client that calss compress() if the command-line
	argument is "-", and expand() if it is "+".
	"""
	if(sys.argv[1] == '-'):
		compress()
	elif(sys.argv[1] == '+'):
		expand()
	else:
		raise ValueError("Illegal command line argument")
	
if __name__ == '__main__':
	main()



# Created for BADS 2018
# See README.md for details
# This is python3 
import math
import random
import sys


#The following part is borrowed from https://langui.sh/2009/03/07/generating-very-large-primes/
#in an effort to implement the missing long_random_prime() function
def _rabin_miller(n):
    s = n-1
    t = 0
    while s&1 == 0:
        s = int(s/2)
        t +=1
    k = 0
    while k<128:
        a = random.randrange(2,n-1)
        #a^s is computationally infeasible.  we need a more intelligent approach
        #v = (a**s)%n
        #python's core math module can do modular exponentiation
        v = pow(a,s,n) #where values are (num,exp,mod)
        if v != 1:
            i=0
            while v != (n-1):
                if i == t-1:
                    return False
                else:
                    i = i+1
                    v = (v**2)%n
        k+=2
    return True
def _is_prime(n):
    #lowPrimes is all primes (sans 2, which is covered by the bitwise and operator)
    #under 1000. taking n modulo each lowPrime allows us to remove a huge chunk
    #of composite numbers from our potential pool without resorting to Rabin-Miller
    lowPrimes =   [3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97
                  ,101,103,107,109,113,127,131,137,139,149,151,157,163,167,173,179
                  ,181,191,193,197,199,211,223,227,229,233,239,241,251,257,263,269
                  ,271,277,281,283,293,307,311,313,317,331,337,347,349,353,359,367
                  ,373,379,383,389,397,401,409,419,421,431,433,439,443,449,457,461
                  ,463,467,479,487,491,499,503,509,521,523,541,547,557,563,569,571
                  ,577,587,593,599,601,607,613,617,619,631,641,643,647,653,659,661
                  ,673,677,683,691,701,709,719,727,733,739,743,751,757,761,769,773
                  ,787,797,809,811,821,823,827,829,839,853,857,859,863,877,881,883
                  ,887,907,911,919,929,937,941,947,953,967,971,977,983,991,997]
    if (n >= 3):
        if (n&1 != 0):
            for p in lowPrimes:
                if (n == p):
                   return True
                if (n % p == 0):
                    return False
            return _rabin_miller(n)
    return False
def long_random_prime(k):
    """
    Generates a random prime.
    :param k: the desired bit length of the prime
    :returns: a random prime of bit length k
    """
    #k is the desired bit length
    r=100*(math.log(k,2)+1) #number of attempts max
    r_ = r
    while r>0:
        #randrange is mersenne twister and is completely deterministic
        #unusable for serious crypto purposes
        n = random.randrange(2**(k-1),2**(k))
        r-=1
        if _is_prime(n) == True:
            return n
    return "Failure after "+ r_ + " tries."
class RabinKarp:
    """
    The RabinKarp class finds the first occurence of a pattern string
    in a text string.
    This implementation uses the Monte Carlo version of the Rabin-Karp algorithm.
    """
    def __init__(self, pat):
        """
        Preprocesses the pattern string.
        :param pat: the pattern string
        """
        self.M = len(pat)
        self.R = 256
        self.Q = long_random_prime(32)
        self.RM = 1
        for i in range (1,self.M):
            self.RM = (self.R * self.RM) % self.Q
        self.patHash = self._hash(pat, self.M)	#pattern hash value
    def search(self, txt):
        """
        Returns the index of the first occurrrence of the pattern string
        in the text string.
        :param txt: the text string
        :return: the index of the first occurrence of the pattern string 
        in the text string; N if no such match
        """
        N = len(txt)
        M = self.M
        RM = self.RM
        Q = self.Q
        R = self.R
        patHash = self.patHash
        txtHash = self._hash(txt, M)
        if(patHash == txtHash):
            return 0
        for i in range(M, N):
        	txtHash = (txtHash + Q - RM *ord(txt[i-M]) % Q) % Q
        	txtHash = (txtHash*R + ord(txt[i])) % Q
        	if(patHash == txtHash):
        		if(self._check(i-M+1)):
        			return i-M+1
        return N
    def _check(self, i):
    #Not needed in the Monte Carlo version
    #For Las Vegas, check pat with txt[i:i-M+1]
        return True
    def _hash(self, key, M):
        #Modular hashing using Horner's method
        h = 0
        for j in range(0,M):
        	h = self.R * h + ord(key[j]) % self.Q
        return h
def main():
	"""
	Takes a pattern string and an input string as command-line arguments;
    searches for the pattern string in the text string; and prints
    the first occurrence of the pattern string in the text string.
    Will print the pattern after the end of the string if no match is found.
	"""
	pat = sys.argv[1]
	txt = sys.argv[2]
	rk = RabinKarp(pat)
	print("text:    {}".format(txt))
	offset = rk.search(txt)
	print("pattern:",end=" ")
	for i in range(0,offset):
		print("",end=" ")
	print(pat)
if __name__ == "__main__":
    main()



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
instream.py

The instream module defines the InStream class.
"""

#-----------------------------------------------------------------------

import re
import sys

if sys.hexversion < 0x03000000:
    import urllib
else:
    import urllib.request as urllib

#-----------------------------------------------------------------------

class InStream:

    """
    An InStream object wraps around a text file or sys.stdin, and
    supports reading from that stream.

    Note:  Usually it's a bad idea to mix these three sets of methods:

    -- isEmpty(), readInt(), readFloat(), readBool(), readString()

    -- hasNextLine(), readLine()
    
    -- readAll(), readAllInts(), readAllFloats(), readAllBools(),
       readAllStrings(), readAllLines()

    Usually it's better to use one set exclusively.
    """

    #-------------------------------------------------------------------

    def __init__(self, fileOrUrl=None):
        """
        Construct self to wrap around a stream. The stream can be
        a file whose name is given as fileOrUrl, a resource whose URL
        is given as fileOrUrl, or sys.stdin by default.
        """
        self._buffer = ''
        self._stream = None
        self._readingWebPage = False

        if fileOrUrl is None:
            from itu.algs4.stdlib import stdio # To change the mode of sys.stdin
            self._stream = sys.stdin
            return

        # Try to open a file, then a URL.
        try:
            if sys.hexversion < 0x03000000:
                self._stream = open(fileOrUrl,'rU')
            else:
                self._stream = open(fileOrUrl, 'r', encoding='utf-8')
        except IOError:
            try:
                self._stream = urllib.urlopen(fileOrUrl)
                self._readingWebPage = True
            except IOError:
                raise IOError('No such file or URL: ' + fileOrUrl)

    #-------------------------------------------------------------------

    def _readRegExp(self, regExp):
        """
        Discard leading white space characters from the stream wrapped
        by self.  Then read from the stream and return a string
        matching regular expression regExp.  Raise an EOFError if no
        non-whitespace characters remain in the stream. Raise a
        ValueError if the next characters to be read from the stream
        do not match regExp.
        """
        if self.isEmpty():
            raise EOFError()
        compiledRegExp = re.compile(r'^\s*' + regExp)
        match = compiledRegExp.search(self._buffer)
        if match is None:
            raise ValueError()
        s = match.group()
        self._buffer = self._buffer[match.end():]
        return s.lstrip()

    #-------------------------------------------------------------------

    def isEmpty(self):
        """
        Return True iff no non-whitespace characters remain in the
        stream wrapped by self.
        """
        while self._buffer.strip() == '':
            line = self._stream.readline()
            if sys.hexversion < 0x03000000 or self._readingWebPage:
                line = line.decode('utf-8')
            if line == '':
                return True
            self._buffer += str(line)
        return False

    #-------------------------------------------------------------------

    def readInt(self):
        """
        Discard leading white space characters from the stream wrapped
        by self.  Then read from the stream a sequence of characters
        comprising an integer.  Convert the sequence of characters to an
        integer, and return the integer.  Raise an EOFError if no
        non-whitespace characters remain in the stream.  Raise a
        ValueError if the next characters to be read from the stream
        cannot comprise an integer.
        """
        s = self._readRegExp(r'[-+]?(0[xX][\dA-Fa-f]+|0[0-7]*|\d+)')
        radix = 10
        strLength = len(s)
        if (strLength >= 1) and (s[0:1] == '0'): radix = 8
        if (strLength >= 2) and (s[0:2] == '-0'): radix = 8
        if (strLength >= 2) and (s[0:2] == '0x'): radix = 16
        if (strLength >= 2) and (s[0:2] == '0X'): radix = 16
        if (strLength >= 3) and (s[0:3] == '-0x'): radix = 16
        if (strLength >= 3) and (s[0:3] == '-0X'): radix = 16
        return int(s, radix)

    #-------------------------------------------------------------------

    def readAllInts(self):
        """
        Read all remaining strings from the stream wrapped by self,
        convert  each to an int, and return those ints in an array.
        Raise a ValueError if any of the strings cannot be converted
        to an int.
        """
        strings = self.readAllStrings()
        ints = []
        for s in strings:
            i = int(s)
            ints.append(i)
        return ints

    #-------------------------------------------------------------------

    def readFloat(self):
        """
        Discard leading white space characters from the stream wrapped
        by self.  Then read from the stream a sequence of characters
        comprising a float.  Convert the sequence of characters to an
        float, and return the float.  Raise an EOFError if no
        non-whitespace characters remain in the stream.  Raise a
        ValueError if the next characters to be read from the stream
        cannot comprise a float.
        """
        s = self._readRegExp(r'[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?')
        return float(s)

    #-------------------------------------------------------------------

    def readAllFloats(self):
        """
        Read all remaining strings from the stream wrapped by self,
        convert each to a float, and return those floats in an array.
        Raise a ValueError if any of the strings cannot be converted
        to a float.
        """
        strings = self.readAllStrings()
        floats = []
        for s in strings:
            f = float(s)
            floats.append(f)
        return floats

    #-------------------------------------------------------------------

    def readBool(self):
        """
        Discard leading white space characters from the stream wrapped
        by self.  Then read from the stream a sequence of characters
        comprising a bool.  Convert the sequence of characters to an
        bool, and return the bool.  Raise an EOFError if no
        non-whitespace characters remain in the stream.  Raise a
        ValueError if the next characters to be read from the stream
        cannot comprise an bool.
        """
        s = self._readRegExp(r'(True)|(False)|1|0')
        if (s == 'True') or (s == '1'):
            return True
        return False

    #-----------------------------------------------------------------------

    def readAllBools(self):
        """
        Read all remaining strings from the stream wrapped by self,
        convert each to a bool, and return those bools in an array.
        Raise a ValueError if any of the strings cannot be converted
        to a bool.
        """
        strings = self.readAllStrings()
        bools = []
        for s in strings:
            b = bool(s)
            bools.append(b)
        return bools


    #-------------------------------------------------------------------

    def readString(self):
        """
        Discard leading white space characters from the stream wrapped
        by self.  Then read from the stream a sequence of characters
        comprising a string, and return the string. Raise an EOFError
        if no non-whitespace characters remain in the stream.
        """
        s = self._readRegExp(r'\S+')
        return s

    #-----------------------------------------------------------------------

    def readAllStrings(self):
        """
        Read all remaining strings from the stream wrapped by self,
        and return them in an array.
        """
        strings = []
        while not self.isEmpty():
            s = self.readString()
            strings.append(s)
        return strings

    #-------------------------------------------------------------------

    def hasNextLine(self):
        """
        Return True iff the stream wrapped by self has a next line.
        """
        if self._buffer != '':
            return True
        else:
            self._buffer = self._stream.readline()
            if sys.hexversion < 0x03000000 or self._readingWebPage:
                self._buffer = self._buffer.decode('utf-8')
            if self._buffer == '':
                return False
            return True

    #-------------------------------------------------------------------

    def readLine(self):
        """
        Read and return as a string the next line of the stream wrapped
        by self.  Raise an EOFError is there is no next line.
        """
        if not self.hasNextLine():
            raise EOFError()
        s = self._buffer
        self._buffer = ''
        return s.rstrip('\n')

    #-------------------------------------------------------------------

    def readAllLines(self):
        """
        Read all remaining lines from the stream wrapped by self, and
        return them as strings in an array.
        """
        lines = []
        while self.hasNextLine():
            line = self.readLine()
            lines.append(line)
        return lines

    #-------------------------------------------------------------------

    def readAll(self):
        """
        Read and return as a string all remaining lines of the stream
        wrapped by self.
        """
        s = self._buffer
        self._buffer = ''
        for line in self._stream:
            if sys.hexversion < 0x03000000 or self._readingWebPage:
                line = line.decode('utf-8')
            s += line
        return s

    #-------------------------------------------------------------------

    def __del__(self):
        """
        Close the stream wrapped by self.
        """
        if self._stream is not None:
            self._stream.close()

#=======================================================================
# For Testing
#=======================================================================

def _main():
    """
    For testing. The first command-line argument should be the name of
    the method that should be called. The optional second command-line
    argument should be the file or URL to read.
    """

    from itu.algs4.stdlib import stdio

    testId = sys.argv[1]
    if len(sys.argv) > 2:
        inStream = InStream(sys.argv[2])
    else:
        inStream = InStream()

    if testId == 'readInt':
        stdio.writeln(inStream.readInt())
    elif testId == 'readAllInts':
        stdio.writeln(inStream.readAllInts())
    elif testId == 'readFloat':
        stdio.writeln(inStream.readFloat())
    elif testId == 'readAllFloats':
        stdio.writeln(inStream.readAllFloats())
    elif testId == 'readBool':
        stdio.writeln(inStream.readBool())
    elif testId == 'readAllBools':
        stdio.writeln(inStream.readAllBools())
    elif testId == 'readString':
        stdio.writeln(inStream.readString())
    elif testId == 'readAllStrings':
        stdio.writeln(inStream.readAllStrings())
    elif testId == 'readLine':
        stdio.writeln(inStream.readLine())
    elif testId == 'readAllLines':
        stdio.writeln(inStream.readAllLines())
    elif testId == 'readAll':
        stdio.writeln(inStream.readAll())

if __name__ == '__main__':
    _main()



# Created for BADS 2018
# See README.md for details
# Python 3

import sys

from itu.algs4.stdlib import stdio


"""
The heap module provides a function for heapsorting an array.
"""


def sort(pq):
    """
    Rearranges the array in ascending order, using the natural order.
    :param pq: the array to be sorted
    """
    n = len(pq)
    for k in range(n//2, 0, -1):
        _sink(pq, k, n)
    while n > 1:
        _exch(pq, 1, n)
        n -= 1
        _sink(pq, 1, n)


def _sink(pq, k, n):
    """
    Moves item at index k down to a legal position on the heap.
    :param k: Index of the item to be moved
    :param n: Amount of items left on the heap
    """
    while 2*k <= n:
        j = 2*k
        if j < n and _less(pq, j, j+1):
            j += 1
        if not _less(pq, k, j):
            break
        _exch(pq, k, j)
        k = j


def _less(pq, i, j):
    """
    Check if item at index i is greater than item at index j on the heap.
    Indices are "off-by-one" to support 1-based indexing
    :param pq: the heap
    :param i: index of the first item
    :param j: index of the second item
    :return: True if item at index i is smaller than item at index j otherwise False
    """
    return pq[i - 1] < pq[j - 1]


def _exch(pq, i, j):
    """
    Exchanges the positions of items at index i and j on the heap.
    Indices are "off-by-one" to support 1-based indexing.
    :param pq: the heap
    :param i: index of the first item
    :param j: index of the second item
    """
    pq[i-1], pq[j-1] = pq[j-1], pq[i-1]


def _show(pq):
    """
    Print the contents of the array
    :param pq: the array to be printed
    """
    for i in range(len(pq)):
        print(pq[i])


def main():
    """
    Reads in a sequence of strings from stdin
    heapsorts them, and prints the result in ascending order.
    """
    a = stdio.readAllStrings()
    sort(a)
    _show(a)


if __name__ == '__main__':
    main()



#!/python ??
# Created for BADS 2018
# see README.md for details
# This is python3 

#     See ResizingArrayBag for a version that uses a resizing array.

from typing import Generic, Iterator, List, Optional, Sequence, TypeVar

T = TypeVar('T')
S = TypeVar('S')

class Bag(Generic[T]):
    """
    The Bag class represents a bag (or multiset) of 
    generic items. It supports insertion and iterating over the 
    items in arbitrary order.

    This implementation uses a singly linked list with a static nested class Node.
    See LinkedBag for the version from the
    textbook that uses a non-static nested class.

    The add, is_empty, and size operations
    take constant time. Iteration takes time proportional to the number of items.
    """
    class Node(Generic[S]):
        # helper linked list class
        def __init__(self):
            self.next: Optional[Node] = None
            self.item: Optional[S] = None

    def __init__(self) -> None:
        """
        Initializes an empty bag.
        """
        self._first: Optional[Bag.Node[T]] = None # beginning of bag
        self._n = 0        # number of elements in bag 

    def is_empty(self) -> bool:
        """
        Returns true if this bag is empty.

        :returns: true if this bag is empty
                  false otherwise
        """
        return self._first is None

    def size(self) -> int:
        """
        Returns the number of items in this bag.

        :returns: the number of items in this bag
        """
        return self._n

    def __len__(self) -> int:
        return self.size()

    def add(self, item: T) -> None:
        """
        Adds the item to this bag.
        
        :param item: the item to add to this bag
        """
        oldfirst = self._first
        self._first = Bag.Node()
        self._first.item = item
        self._first.next = oldfirst
        self._n += 1

    def __iter__(self) -> Iterator[T]:
        """
        Returns an iterator that iterates over the items in this bag in arbitrary order.

        :returns: an iterator that iterates over the items in this bag in arbitrary order
        """
        current = self._first
        while current is not None:
            assert current.item is not None
            yield current.item
            current = current.next

    def __repr__(self) -> str:
        out = '{'
        for elem in self:
            out += '{}, '.format(elem)
        return out + '}'


# start of the script itself
if __name__ == '__main__':
    import sys
    from itu.algs4.stdlib import stdio
    
    if len(sys.argv) > 1:
        try:
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")    

    bag: Bag[str] = Bag()
    while not stdio.isEmpty():
        item = stdio.readString()
        bag.add(item)

    stdio.writef("size of bag = %i\n", bag.size())
    
    for s in bag:
        stdio.writeln(s)



import sys

from itu.algs4.errors.errors import IllegalArgumentException
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph
from itu.algs4.sorting.index_min_pq import IndexMinPQ
from itu.algs4.stdlib.instream import InStream

# Created for BADS 2018
# See README.md for details
# Python 3


class DijkstraSP:
    """
    The DijkstraSP class represents a data type for solving the
    single-source shortest paths problem in edge-weighted digraphs
    where the edge weights are nonnegative.
    This implementation uses Dijkstra's algorithm with a binary heap.
    The constructor takes time proportional to E log V,
    where V is the number of vertices and E is the number of edges.
    Each call to dist_to() and has_path_to() takes constant time.
    Each call to path_to() takes time proportional to the number of
    edges in the shortest path returned.
    """
    def __init__(self, G, s):
        """
        Computes a shortest-paths tree from the source vertex s to every other
        vertex in the edge-weighted digraph G.
        :param G: The edge-weighted digraph
        :param s: The source vertex
        :raises IllegalArgumentException: if an edge weight is negative
        :raises IllegalArgumentException: unless 0 <= s < V
        """
        for e in G.edges():
            if e.weight() < 0:
                raise IllegalArgumentException("edge {} has negative weight".format(e))
        self._dist_to = [float('inf')] * G.V()
        self._edge_to = [None] * G.V()
        self._validate_vertex(s)
        self._dist_to[s] = 0.0
        self._pq = IndexMinPQ(G.V())
        self._pq.insert(s, 0.0)
        while not self._pq.is_empty():
            v = self._pq.del_min()
            for e in G.adj(v):
                self._relax(e)

    def dist_to(self, v):
        """
        Returns the length of a shortest path from the source vertex s to vertex v.
        :param v: the destination vertex
        :return: the length of a shortest path from the source vertex s to vertex v
        :rtype: float
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._dist_to[v]

    def has_path_to(self, v):
        """
        Returns True if there is a ath from the source vertex s to vertex v.
        :param v: the destination vertex
        :return: True if there is a path from the source vertex
        s to vertex v. Otherwise returns False
        :rtype: bool
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._dist_to[v] < float('inf')

    def path_to(self, v):
        """
        Returns a shortest path from the source vertex s to vertex v.
        :param v: the destination vertex
        :return: a shortest path from the source vertex s to vertex v
        :rtype: collections.iterable[DirectedEdge]
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        if not self.has_path_to(v):
            return None
        path = Stack()
        e = self._edge_to[v]
        while e is not None:
            path.push(e)
            e = self._edge_to[e.from_vertex()]
        return path

    def _relax(self, e):
        """
        Relaxes the edge e and updates the pq if changed.
        :param e: the edge to relax
        """
        v = e.from_vertex()
        w = e.to_vertex()
        if self._dist_to[w] > self._dist_to[v] + e.weight():
            self._dist_to[w] = self._dist_to[v] + e.weight()
            self._edge_to[w] = e
            if self._pq.contains(w):
                self._pq.decrease_key(w, self._dist_to[w])
            else:
                self._pq.insert(w, self._dist_to[w])

    def _validate_vertex(self, v):
        """
        Raises an IllegalArgumentException unless 0 <= v < V
        :param v: the vertex to be validated
        """
        V = len(self._dist_to)
        if v < 0 or v >= V:
            raise IllegalArgumentException("vertex {} is not between 0 and {}".format(v, V-1))


def main():
    """
    Creates an EdgeWeightedDigraph from input file.
    Runs DijkstraSP on the graph with the given source vertex.
    Prints the shortest path from the source vertex to all other vertices.
    """
    if len(sys.argv) == 3:
        stream = InStream(sys.argv[1])
        G = EdgeWeightedDigraph.from_stream(stream)
        s = int(sys.argv[2])
        sp = DijkstraSP(G, s)
        for t in range(G.V()):
            if sp.has_path_to(t):
                print("{} to {} ({:.2f})  ".format(s, t, sp.dist_to(t)), end='')
                for e in sp.path_to(t):
                    print(e, end='   ')
                print()
            else:
                print("{} to {}         no path\n".format(s, t))


if __name__ == '__main__':
    main()



# Created for BADS 2018
# see README.md for details
# This is python3 

from itu.algs4.fundamentals.stack import Stack


class Cycle:
    """
    The Cycle class represents a data type for 
    determining whether an undirected graph has a cycle.
    The hasCycle operation determines whether the graph has
    a cycle and, if so, the cycle operation returns one.

    This implementation uses depth-first search.
    The constructor takes time proportional to V + E
    (in the worst case),
    where V is the number of vertices and E is the number of edges.
    Afterwards, the hasCycle operation takes constant time
    the cycle operation takes time proportional
    to the length of the cycle.
    """

    def __init__(self, G):
        """
        Determines whether the undirected graph G has a cycle and,
        if so, finds such a cycle.
     
        :param G: the undirected graph
        """
        if self._has_self_loop(G): return
        if self._has_parallel_edges(G): return
        self._marked = [False] * G.V()
        self._edgeTo = [0] * G.V()
        self._cycle = None
        for v in range(G.V()):
            if not self._marked[v]:
                self._dfs(G, -1, v)

    def _has_self_loop(self, G):
        # does this graph have a self loop?
        # side effect: initialize cycle to be self loop
        for v in range(G.V()):
            for w in G.adj(v):
                if v == w:
                    self._cycle = Stack()
                    self._cycle.push(v)
                    self._cycle.push(w)
                    return True
        return False

    def _has_parallel_edges(self, G):
        # does this graph have two parallel edges?
        # side effect: initialize cycle to be two parallel edges
        self._marked = [False] * G.V()
        for v in range(G.V()):
            # check for parallel edges incident to v
            for w in G.adj(v):
                if self._marked[w]:
                    self._cycle = Stack()
                    self._cycle.push(v)
                    self._cycle.push(w)
                    self._cycle.push(v)
                    return True
                self._marked[w] = True

            for w in G.adj(v):
                self._marked[w] = False
        return False

    def has_cycle(self):
        """
        Returns true if the graph G has a cycle.

        :returns: true if the graph has a cycle false otherwise
        """
        return self._cycle is not None   
    
    def cycle(self):
        """
        Returns a cycle in the graph G.
        
        :returns: a cycle if the graph G has a cycle,
            and null otherwise
        """
        return self._cycle
    
    def _dfs(self, G, u, v):
        self._marked[v] = True
        for w in G.adj(v):
            # short circuit if cycle already found
            if self._cycle is not None: return
            if not self._marked[w]:
                self._edgeTo[w] = v
                self._dfs(G, v, w)
            elif w != u:
                self._cycle = Stack()
                x = v
                while x != w:
                    self._cycle.push(x)
                    x = self._edgeTo[x]
                self._cycle.push(w)
                self._cycle.push(v)

if __name__ == "__main__":
    import sys
    from itu.algs4.stdlib.instream import InStream
    from itu.algs4.stdlib import stdio
    from itu.algs4.graphs.graph import Graph    

    In = InStream(sys.argv[1])
    G = Graph.from_stream(In)
    finder = Cycle(G)
    if finder.has_cycle():
        for v in finder.cycle():
            stdio.writef("%i ", v)
        stdio.writeln()
    else:
        stdio.writeln("Graph is acyclic")



# Created for BADS 2018
# See README.md for details
# Python 3

import math
import sys

from itu.algs4.graphs.bellman_ford_sp import BellmanFordSP
from itu.algs4.graphs.directed_edge import DirectedEdge
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph
from itu.algs4.stdlib import stdio

if __name__ == '__main__':
    """
    The Arbitrage function provides a client that finds an arbitrage opportunity 
    in a currency exchange table by constructing a complete-digraph representation 
    of the exchange table and then finding a negative cycle in the digraph.
    
    This implementation uses the Bellman-Ford algorithm to find a negative cycle in 
    the complete digraph. The running time is proportional to V3 in the worst case, 
    where V is the number of currencies.

    For additional documentation, see Section 4.4 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """
    if len(sys.argv) > 1:
        try: 
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")
    
    # V currencies
    V = stdio.readInt()
    name = [None]*V
    
    # Create complete network
    graph = EdgeWeightedDigraph(V)
    for v in range(V):
        name[v] = stdio.readString()
        for w in range(V):
            rate = stdio.readFloat()
            edge = DirectedEdge(v, w, -math.log(rate))
            graph.add_edge(edge)
    
    # find negative cycle
    spt = BellmanFordSP(graph, 0)
    if spt.has_negative_cycle():
        stake = 1000.0
        for edge in spt.negative_cycle():
            print('{} {}', stake, name[edge.from_vertex()])
            stake *= math.exp(-edge.weight())
            print('{} {}')



# Created for BADS 2018
# See README.md for details
# This is python3 
import sys


"""
The KMP (Knuth-Morris-Pratt) class finds the first occurrence 
of a pattern string in a text string.
This implementation uses a version of the Knuth-Morris-Pratt substring search
algorithm. The version takes time as space proportional to
N + M R in the worst case, where N is the length
of the text string, M is the length of the pattern, and R
is the alphabet size.
"""
class KMP:
	def __init__(self, pat):
		"""
    	Preprocesses the pattern string.
    	:param pat: the pattern string
    	"""
		self.pat = pat
		M = len(pat)
		R = 256
		self.dfa = [[0 for c in range(0,M)] for r in range(0,R)]
		self.dfa[ord(pat[0])][0] = 1
		X = 0
		for j in range (1,M):
			#Compute dfa[][j]
			for c in range(0,R):
				self.dfa[c][j] = self.dfa[c][X] #Copy mismatch cases
			self.dfa[ord(pat[j])][j] = j+1 #Set match case
			X = self.dfa[ord(pat[j])][X] #Update restart state
	def search(self, txt):
		"""
		Returns the index of the first occurrrence of the pattern string
     	in the text string.
     	:param txt: the text string
     	:return: the index of the first occurrence of the pattern string 
     	in the text string; N if no such match
		"""
		N = len(txt)
		M = len(self.pat)
		j = 0
		i = 0
		while(i < N and j < M):
			j = self.dfa[ord(txt[i])][j]
			i += 1
		if(j==M): #found (hit end of pattern)
			return i-M 
		else: #not found (hit end of text)
			return N
def main():
	"""
	Takes a pattern string and an input string as command-line arguments;
    searches for the pattern string in the text string; and prints
    the first occurrence of the pattern string in the text string.
    Will print the pattern after the end of the string if no match is found.
	"""
	pat = sys.argv[1]
	txt = sys.argv[2]
	kmp = KMP(pat)
	print("text:    {}".format(txt))
	offset = kmp.search(txt)
	print("pattern: ",end='')
	for i in range(0,offset):
		print("",end=' ')
	print(pat)
	
if __name__ == "__main__":
	main()



import sys

from itu.algs4.errors.errors import IllegalArgumentException
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.edge_weighted_graph import EdgeWeightedGraph
from itu.algs4.sorting.index_min_pq import IndexMinPQ
from itu.algs4.stdlib.instream import InStream

# Created for BADS 2018
# See README.md for details
# Python 3


class DijkstraUndirectedSP:
    """
    The DijkstraSP class represents a data type for solving the
    single-source shortest paths problem in edge-weighted diagraphs
    where the edge weights are nonnegative.
    This implementation uses Dijkstra's algorithm with a binary heap.
    The constructor takes time proportional to E log V,
    where V is the number of vertices and E is the number of edges.
    Each call to dist_to() and has_path_to() takes constant time
    each call to path_to() takes time proportional to the number of
    edges in the shortest path returned.
    """
    def __init__(self, G, s):
        """
        Computes a shortest-paths tree from the source vertex s to every
        other vertex in the edge-weighted graph G.
        :param G: the edge-weighted graph
        :param s: the source vertex
        :raises IllegalArgumentException: if an edge weight is negative
        :raises IllegalArgumentException: unless 0 <= s < V
        """
        for e in G.edges():
            if e.weight() < 0:
                raise IllegalArgumentException("edge {} has negative weight".format(e))

        self._dist_to = [float('inf')] * G.V()
        self._edge_to = [None] * G.V()
        self._dist_to[s] = 0.0
        self._validate_vertex(s)
        self._pq = IndexMinPQ(G.V())
        self._pq.insert(s, 0)

        while not self._pq.is_empty():
            v = self._pq.del_min()
            for e in G.adj(v):
                self._relax(e, v)

    def dist_to(self, v):
        """
        Returns the length of a shortest path between the source vertex s and
        vertex v.
        :param v: the destination vertex
        :return: the length of a shortest path between the source vertex s and
        the vertex v. float('inf') is not such path
        :rtype: float
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        return self._dist_to[v]

    def has_path_to(self, v):
        """
        Returns true if there is a path between the source vertex s and
        vertex v.
        :param v: the destination vertex
        :return: True if there is a path between the source vertex
        s to vertex v. False otherwise
        :rtype: bool
        """
        return self._dist_to[v] < float('inf')

    def path_to(self, v):
        """
        Returns a shortest path between the source vertex s and vertex v.
        :param v: the destination vertex
        :return: a shortest path between the source vertex s and vertex v.
        None if no such path
        :rtype: collections.iterable[Edge]
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        if not self.has_path_to(v):
            return None
        path = Stack()
        x = v
        e = self._edge_to[v]
        while e is not None:
            edge = self._edge_to[x]
            path.push(edge)
            x = e.other(x)
            e = self._edge_to[x]
        return path

    def _validate_vertex(self, v):
        """
        Raises an IllegalArgumentException unless 0 <= v < V
        :param v: the vertex to validate
        """
        V = len(self._dist_to)
        if v < 0 or v >= V:
            raise IllegalArgumentException("vertex {} is not between 0 and {}".format(v, V-1))

    def _relax(self, e, v):
        """
        Relax edge e and update pq if changed
        :param e: the edge to relax
        :param v: the vertex e goes out from
        """
        w = e.other(v)
        if self._dist_to[v] + e.weight() < self._dist_to[w]:
            self._dist_to[w] = self._dist_to[v] + e.weight()
            self._edge_to[w] = e
            if self._pq.contains(w):
                self._pq.decrease_key(w, self._dist_to[w])
            else:
                self._pq.insert(w, self._dist_to[w])


def main():
    if len(sys.argv) == 3:
        stream = InStream(sys.argv[1])
        G = EdgeWeightedGraph.from_stream(stream)
        s = int(sys.argv[2])
        sp = DijkstraUndirectedSP(G, s)

        for t in range(G.V()):
            if sp.has_path_to(t):
                print("{} to {} ({:.2f})  ".format(s, t, sp.dist_to(t)), end='')
                for e in sp.path_to(t):
                    print(e, end='   ')
                print()
            else:
                print("{} to {}         no path\n".format(s, t))


if __name__ == '__main__':
    main()



# Created for BADS 2018
# see README.md for details
# This is python3 

class SequentialSearchST:
    """
    The  SequentialSearchST class represents an (unordered)
    symbol table of generic key-value pairs.
    It supports the usual put, get, contains,
    delete, size, and is-empty methods.
    It also provides a keys method for iterating over all of the keys.
    A symbol table implements the associative array abstraction:
    when associating a value with a key that is already in the symbol table,
    the convention is to replace the old value with the new value.
    The class also uses the convention that values cannot be  None. Setting the
    value associated with a key to  None is equivalent to deleting the key
    from the symbol table.

    This implementation uses a singly-linked list and sequential search.
    It relies on the  equals() method to test whether two keys
    are equal. It does not call either the  compareTo() or
    hashCode() method. 
    The put and delete operations take linear time the
    get and contains operations takes linear time in the worst case.
    The size, and is-empty operations take constant time.
    Construction takes constant time.
    """
    
    class Node:
        # a helper linked list data type
        def __init__(self, key, val, next):
            self.key = key
            self.val = val
            self.next = next

    def __init__(self):
        """Initializes an empty symbol table."""
        self._n = 0         # number of key-value pairs
        self._first = None  # the linked list of key-value pairs


    def size(self):
        """
        Returns the number of key-value pairs in this symbol table.

        :returns: the number of key-value pairs in this symbol table
        """
        return self._n

    def __len__(self):
        return self.size()

    def is_empty(self):
        """
        Returns true if this symbol table is empty.
        
        :returns:  true if this symbol table is empty
                 false otherwise
        """
        return self.size() == 0

    def contains(self, key):        
        """"    
        Returns true if this symbol table contains the specified key.

        :param  key the key
        :returns: true if this symbol table contains key
                  false otherwise
        :raises ValueError: if key is None
        """
        if key is None: raise ValueError("argument to contains() is None")
        return self.get(key) is not None

    def get(self, key):
        """
        Returns the value associated with the given key in this symbol table.
        
        :param key: the key
        :returns: the value associated with the given key if the key is in the symbol table
                    and None if the key is not in the symbol table
        :raises ValueError: if key is None
        """
        if key is None: raise ValueError("argument to get() is None")
        x = self._first
        while x is not None:
            if key == x.key:
                return x.val
            x = x.next
        return None
    
    def put(self, key, val):
        """
        Inserts the specified key-value pair into the symbol table, overwriting the old 
        value with the new value if the symbol table already contains the specified key.
        Deletes the specified key (and its associated value) from this symbol table
        if the specified value is None.
        
        :param key: the key
        :param val: the value
        :raises ValueError: if key is None
        """
        if key is None: raise ValueError("argument to put() is None")
        if val is None:
            self.delete(key)
            return

        x = self._first
        while x is not None:
            if key == x.key:
                x.val = val
                return
            x = x.next

        self._first = self.Node(key, val, self._first)
        self._n += 1

    def delete(self, key):
        """
        Removes the specified key and its associated value from this symbol table     
        (if the key is in this symbol table).    
                
        :param  key the key
        :raises ValueError: if key is None
        """
        if key is None: raise ValueError("argument to delete() is None")
        self._first = self._delete(self._first, key)

    def _delete(self, x, key):
        # delete key in linked list beginning at Node x
        # warning: function call stack too large if table is large
        if x is None: return None
        if key == x.key:
            self._n -= 1
            return x.next

        x.next = self._delete(x.next, key)
        return x

    def keys(self):
        """
        Returns all keys in the symbol table as an  Iterable.
        To iterate over all of the keys in the symbol table named  st,
        use the foreach notation: for Key key in st.keys().
     
        :returns: all keys in the symbol table
        """
        x = self._first
        while x is not None:
            yield x.key
            x = x.next


if __name__ == "__main__":
    from itu.algs4.stdlib import stdio

    st = SequentialSearchST()
    i = 0
    while not stdio.isEmpty():
        key = stdio.readString()
        st.put(key, i)
        i += 1
        
    for s in st.keys():
        stdio.writef("%s %i\n", s, st.get(s))



import sys

from itu.algs4.errors.errors import IllegalArgumentException
from itu.algs4.fundamentals.bag import Bag
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.edge import Edge
from itu.algs4.stdlib.instream import InStream

# Created for BADS 2018
# See README.md for details
# Python 3


class EdgeWeightedGraph:
    """
    The EdgeWeightedGraph class represents an edge-weighted
    graph of vertices named 0 through V-1, where each
    undirected edge is of type Edge and has a real-valued weight.
    It supports the following two primary operations: add an edge to the graph,
    iterate over all of the edges incident to a vertex. It also provides
    methods for returning the number of vertices V and the number
    of edges E. Parallel edges and self-loops are permitted.
    By convention, a self-loop v-v appears in the adjacency list of v twice and contributes two to the degree
    of v.
    This implementation uses an adjacency-list representation, which
    is a vertex-indexed array of Bag objects.
    All operations take constant time (in the worst case) except
    iterating over the edges incident to a given vertex, which takes
    time proportional to the number of such edges.
    """
    def __init__(self, V):
        """
        Initializes an empty edge-weighted graph with V vertices and 0 edges.
        :param V: the number of vertices
        :raises IllegalArgumentException: if V < 0
        """
        if V < 0:
            raise IllegalArgumentException("Number of vertices must be nonnegative")
        self._V = V
        self._E = 0
        self._adj = [None] * V
        for v in range(V):
            self._adj[v] = Bag()

    @staticmethod
    def from_graph(G):
        """
        Initializes a new edge-weighted graph that is a deep copy of G.
        :param G: the edge-weighted graph to copy
        :return: the copy of the graph edge-weighted graph G
        :rtype: EdgeWeightedGraph
        """
        g = EdgeWeightedGraph(G.V())
        g._E = G.E()
        for v in range(G.V()):
            reverse = Stack()
            for e in G.adj(v):
                reverse.push(e)
            for e in reverse:
                g._adj[v].add(e)
        return g

    @staticmethod
    def from_stream(stream):
        """
        Initializes an edge-weighted graph from an input stream.
        The format is the number of vertices V,
        followed by the number of edges E,
        followed by E pairs of vertices and edge weights,
        with each entry separated by whitespace.
        :param stream: the input stream
        :raises IllegalArgumentException: if the endpoints of any edge are not in prescribed range
        :raises IllegalArgumentException: if the number of vertices or edges is negative
        :return: the edge-weighted graph
        :rtype: EdgeWeightedGraph
        """
        g = EdgeWeightedGraph(stream.readInt())
        E = stream.readInt()
        if E < 0:
            raise IllegalArgumentException("Number of edges must be nonnegative")
        for i in range(E):
            v = stream.readInt()
            w = stream.readInt()
            g._validate_vertex(v)
            g._validate_vertex(w)
            weight = stream.readFloat()
            e = Edge(v, w, weight)
            g.add_edge(e)
        return g

    def add_edge(self, e):
        """
        Adds the undirected edge e to this edge-weighted graph.
        :param e: the edge
        """
        v = e.either()
        w = e.other(v)
        self._validate_vertex(v)
        self._validate_vertex(w)
        self._adj[v].add(e)
        self._adj[w].add(e)
        self._E += 1

    def adj(self, v):
        """
        Returns the edges incident on vertex v.
        :param v: the vertex
        :return: the edges incident on vertex v
        :rtype: collections.iterable[Edge]
        """
        self._validate_vertex(v)
        return self._adj[v]

    def V(self):
        """
        Returns the number of vertices in this edge-weighted graph.
        :return: the number of vertices in this edge-weighted graph
        :rtype: int
        """
        return self._V

    def E(self):
        """
        Returns the number of edges in this edge-weighted graph.
        :return: the number of edges in this edge-weighted graph
        :rtype: int
        """
        return self._E

    def degree(self, v):
        """
        Returns the degree of vertex v.
        :param v: the vertex
        :return: the degree of vertex v
        :rtype: int
        :raises IllegalArgumentException: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._adj[v].size()

    def edges(self):
        """
        Returns all edges in this edge-weighted graph.
        :return: all edges in this edge-weighted graph
        """
        edges = Bag()
        for v in range(self._V):
            self_loops = 0
            for e in self.adj(v):
                if e.other(v) > v:
                    edges.add(e)
                elif e.other(v) is v:
                    if self_loops % 2 is 0:
                        edges.add(e)
                    self_loops += 1
        return edges

    def _validate_vertex(self, v):
        """
        Raises an IllegalArgumentException unless 0 <= v < V.
        :param v: the vertex to be validated
        """
        if v < 0 or v >= self._V:
            raise IllegalArgumentException("vertex {} is not between 0 and {}".format(v, self._V-1))

    def __repr__(self):
        """
        Returns a string representation of the edge-weighted graph.
        This method takes time proportional to E + V.
        :return: the number of vertices, followed by the number of edges,
        followed by the V adjacency lists of edges
        """
        s = ["{} {} \n".format(self._V, self._E)]
        for v in range(self._V):
            s.append("{}: ".format(v))
            for e in self._adj[v]:
                s.append("{}: ".format(e))
            s.append("\n")
        return ''.join(s)


def main():
    """
    Creates an edge-weighted graph from the given input file and prints it.
    """
    if len(sys.argv) > 1:
        stream = InStream(sys.argv[1])
        G = EdgeWeightedGraph.from_stream(stream)
        print(G)


if __name__ == '__main__':
    main()



# Created for BADS 2018
# See README.md for details
# This is python3 
"""
The LSW module provides static methods for compressing and
expanding a binary input using LZW over the 8-bit extended
ASCII alphabet with 12-bit codewords.

For additional documentation see Section 5.5 of
Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
"""
import sys

from itu.algs4.stdlib.binary_stdin import BinaryStdIn
from itu.algs4.stdlib.binary_stdout import BinaryStdOut
from itu.algs4.strings.tst import TST

_R = 256
_L = 4096
_W = 12
def compress():
	"""
	Reads a sequence of 8-bit bytes from standard input; compresses
	them using LZW compression with 12-bit codewords; and writes the results
	to standard output.
	"""
	input_ = BinaryStdIn.read_string()
	st = TST()
	for i in range(0,_R):
		st.put(""+chr(i),i)
	code = _R+1
	while(len(input_) > 0):
		s = st.longest_prefix_of(input_)
		BinaryStdOut.write_int(st.get(s),_W)
		t = len(s)
		if(t < len(input_) and code < _L):
			st.put(input_[0:t+1],code)
			code += 1
		input_ = input_[t:]
	BinaryStdOut.write_int(_R,_W)
	BinaryStdOut.close()
	
def expand():
	"""
	Reads a sequence of bit encoded using LZW compression with
	12-bit codewords from standard input; expands them; and writes
	the results to standard output.
	"""
	st = ["" for i in range(0,_L)]
	i = 0
	while(i < _R):
		st[i] = ""+chr(i)
		i += 1
	st[i] = ""
	i += 1

	codeword = BinaryStdIn.read_int(_W)
	if(codeword == _R):
		return
	val = st[codeword]
	while(True):
		BinaryStdOut.write_string(val)
		codeword = BinaryStdIn.read_int(_W)
		if(codeword == _R):
			break
		s = st[codeword]
		if(i == codeword):
			s = val + val[0]
		if(i < _L):
			st[i] = val+s[0]
			i+=1
		val = s
	BinaryStdOut.close()
	
def main():
	"""
	Sample client that calls compress() if the command-line
	argument is "-", and expand() if it is "+".

        Example: echo huhu | python3 algs4/strings/lzw.py - | python3 algs4/strings/lzw.py +
	"""
	if(sys.argv[1] == '-'):
		compress()
	elif(sys.argv[1] == '+'):
		expand()
	else:
		raise ValueError("Illegal command line argument")
	
if __name__ == '__main__':
	main()



# Created for BADS 2018
# See README.md for details
# Python 3

import math
import sys

from itu.algs4.fundamentals.stack import Stack
from itu.algs4.stdlib import stdio


def evaluate():
    ops = Stack()
    vals = Stack()

    while not stdio.isEmpty():
        # Read token, push if operator
        s = stdio.readString()
        if   s == "(": pass
        elif s == "+":      ops.push(s)
        elif s == "-":      ops.push(s)
        elif s == "*":      ops.push(s)
        elif s == "/":      ops.push(s)
        elif s == "sqrt":   ops.push(s)
        elif s == ")":
            # Pop, evaluate and push result if token is ")"
            op = ops.pop()
            v = vals.pop()
            if   op == "+":     v = vals.pop() + v
            elif op == "-":     v = vals.pop() - v
            elif op == "*":     v = vals.pop() * v
            elif op == "/":     v = vals.pop() / v
            elif op == "sqrt":  v = math.sqrt(v)
            vals.push(v)
        else:   
            vals.push(float(s))
    stdio.writeln(vals.pop())

if __name__ == "__main__":
    if len(sys.argv) > 1:
        try:
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")
            
    evaluate()



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
color.py

The color module defines the Color class and some popular Color
objects.
"""

#-----------------------------------------------------------------------

class Color:
    """
    A Color object models an RGB color.
    """

    #-------------------------------------------------------------------

    def __init__(self, r=0, g=0, b=0):
        """
        Construct self such that it has the given red (r),
        green (g), and blue (b) components.
        """
        self._r = r  # Red component
        self._g = g  # Green component
        self._b = b  # Blue component

    #-------------------------------------------------------------------

    def getRed(self):
        """
        Return the red component of self.
        """
        return self._r

    #-------------------------------------------------------------------

    def getGreen(self):
        """
        Return the green component of self.
        """
        return self._g

    #-------------------------------------------------------------------

    def getBlue(self):
        """
        Return the blue component of self.
        """
        return self._b

    #-------------------------------------------------------------------


    def __str__(self):
        """
        Return the string equivalent of self, that is, a
        string of the form '(r, g, b)'.
        """
        #return '#%02x%02x%02x' % (self._r, self._g, self._b)
        return '(' + str(self._r) + ', ' + str(self._g) + ', ' + \
            str(self._b) + ')'

#-----------------------------------------------------------------------

# Some predefined Color objects:

WHITE      = Color(255, 255, 255)
BLACK      = Color(  0,   0,   0)

RED        = Color(255,   0,   0)
GREEN      = Color(  0, 255,   0)
BLUE       = Color(  0,   0, 255)

CYAN       = Color(  0, 255, 255)
MAGENTA    = Color(255,   0, 255)
YELLOW     = Color(255, 255,   0)

DARK_RED   = Color(128,   0,   0)
DARK_GREEN = Color(  0, 128,   0)
DARK_BLUE  = Color(  0,   0, 128)

GRAY       = Color(128, 128, 128)
DARK_GRAY  = Color( 64,  64,  64)
LIGHT_GRAY = Color(192, 192, 192)

ORANGE     = Color(255, 200,   0)
VIOLET     = Color(238, 130, 238)
PINK       = Color(255, 175, 175)

# Shade of blue used in Introduction to Programming in Java.
# It is Pantone 300U. The RGB values are approximately (9, 90, 166).
BOOK_BLUE  = Color(  9,  90, 166)
BOOK_LIGHT_BLUE = Color(103, 198, 243)

# Shade of red used in Algorithms 4th edition
BOOK_RED   = Color(150,  35,  31)

#-----------------------------------------------------------------------

def _main():
    """
    For testing:
    """
    from itu.algs4.stdlib import stdio
    c1 = Color(128, 128, 128)
    stdio.writeln(c1)
    stdio.writeln(c1.getRed())
    stdio.writeln(c1.getGreen())
    stdio.writeln(c1.getBlue())

if __name__ == '__main__':
    _main()



# created for BADS 2018
# see README.md for details
# Python 3

import sys

from itu.algs4.fundamentals.queue import Queue
from itu.algs4.stdlib import stdio

try:
    q = Queue()
    q.enqueue(1)
except AttributeError:
    print('ERROR - unable to import python algs4 Queue class')
    sys.exit(1)
 
 # Symbol table with string keys, implemented using a ternary search
 # trie (TST).
 
"""
  The TST class represents an symbol table of key-value
  pairs, with string keys and generic values.
  It supports the usual put, get, contains,
  delete, size, and is-empty methods.
  It also provides character-based methods for finding the string
  in the symbol table that is the longest prefix of a given prefix,
  finding all strings in the symbol table that start with a given prefix,
  and finding all strings in the symbol table that match a given pattern.
  A symbol table implements the associative array abstraction:
  when associating a value with a key that is already in the symbol table,
  the convention is to replace the old value with the new value.
  This class uses the convention that
  values cannot be None setting the
  value associated with a key to None is equivalent to deleting the key
  from the symbol table.
  This implementation uses a ternary search trie.
"""

class TST(object):
    class Node():
        def __init__(self):
            self.c = None       # character
            self.left = None    # left, middle and right subtries
            self.mid = None
            self.right = None
            self.val = None     # value associated with string

    def __init__(self):
        self.n = 0         # size
        self.root = None   # root of TST

    
    # @return the number of key-value pairs in this symbol table
    def size(self):
        return self.n

    def __len__(self):
        return self.size()

    # Does this symbol table contain the given key?
    # @param key the key
    # @return True if this symbol table contains key and
    #     False otherwise
    # @raises ValueError if key is None
    def contains(self, key):
        if key is None:
            raise ValueError("argument of contains is None") # TODO maybe get a specific exception, like IllegalArgumentException in Java
        return self.get(key) is not None

    # Returns the value associated with the given key.
    # @param key the key
    # @return the value associated with the given key if the key is in the symbol table
    #     and null if the key is not in the symbol table
    # @raises ValueError if key is None
    def get(self, key):
        if key is None:
            raise ValueError("calls get() with null argument" ) # TODO IllegalArgumentException?
        if len(key) == 0:
            raise ValueError("key must have length >=1") # TODO IllegalArgumentException?
        x = self._get(self.root, key, 0)
        if x is None:
            return None
        return x.val

    # return subtrie corresponding to given key
    def _get(self, x, key, d):
        if x is None:
            return None
        if len(key) == 0:
            raise ValueError("key nust have length >= 1")
        c = key[d]
        if c < x.c: 
            return self._get(x.left, key, d)
        elif c > x.c:
            return self._get(x.right, key, d)
        elif d < len(key) -1:
            return self._get(x.mid, key, d+1)
        else:
            return x

    # Inserts the key-value pair into the symbol table, overwriting the old value
    # with the new value if the key is already in the symbol table.
    # If the value is None, this effectively deletes the key from the symbol table.
    # @param key the key
    # @param val the value
    # @raises ValueError if key is None
    
    def put(self, key, val):
        if key is None:
            raise ValueError("calls put() with null key") # TODO IllegalArgumentException 
        if not self.contains(key):
            self.n += 1
        self.root = self._put(self.root, key, val, 0)

    def _put(self, x, key, val, d):
        c = key[d]
        if x is None:
            x = self.Node()
            x.c = c
        if c < x.c:
            x.left = self._put(x.left, key, val, d)
        elif c > x.c:
            x.right = self._put(x.right, key, val, d)
        elif d < len(key) -1:
            x.mid = self._put(x.mid, key, val, d+1)
        else:
            x.val = val

        return x

    # Returns the string in the symbol table that is the longest prefix of query,
    # or None, if no such string.
    # @param query the query string
    # @return the string in the symbol table that is the longest prefix of query,
    #     or None if no such string
    # @raises ValueError if query is None
    
    def longest_prefix_of(self, query):
        if query is None:
            raise ValueError("calls longest_path_of() with None argument")
        if len(query) == 0:
            return None
        length = 0
        x = self.root
        i = 0
        while (x is not None and i < len(query)):
            c = query[i]
            if c < x.c:
                x = x.left
            elif c > x.c:
                x = x.right
            else:
                i += 1
                if x.val is not None:
                    length = i
                x = x.mid
        return query[0:length]

    # Returns all keys in the symbol table as an Iterable.
    # To iterate over all of the keys in the symbol table named st,
    # use the foreach notation: for key in st.keys().
    # @return all keys in the symbol table as an Iterable
    
    def keys(self):
        queue = Queue()
        self._collect(self.root, "", queue)
        return queue
        
    # Returns all of the keys in the set that start with prefix.
    # @param prefix the prefix
    # @return all of the keys in the set that start with prefix,
    #     as an iterable
    # @raises ValueError if prefix is None
    
    def keys_with_prefix(self, prefix):
        if prefix is None:
            raise ValueError("calls keys_with_prefix with null argument") # TODO IllegalArgumentException
        queue = Queue()
        x = self._get(self.root, prefix, 0)
        if x is None:
            return queue
        if x.val is not None:
            queue.enqueue(prefix)
        self._collect(x.mid, prefix, queue)
        return queue

    # all keys in subtrie rooted at x with given prefix
    def _collect(self, x, prefix, queue):
        if x is None:
            return
        self._collect(x.left, prefix, queue)
        if x.val is not None:
            queue.enqueue(prefix + str(x.c))
        self._collect(x.mid, prefix + str(x.c), queue)
        self._collect(x.right, prefix, queue)
    
    # Returns all of the keys in the symbol table that match pattern,
    # where . symbol is treated as a wildcard character.
    # @param pattern the pattern
    # @return all of the keys in the symbol table that match pattern,
    #     as an iterable, where . is treated as a wildcard character.
    
    def keys_that_match(self, pattern):
        queue = Queue()
        self._collect_match(self.root, "", 0, pattern, queue)
        return queue
        
    def _collect_match(self, x, prefix, i, pattern, queue):
        if x is None:
            return
        c = pattern[i] 
        if c == '.' or c <x.c:
            self._collect_match(x.left, prefix, i, pattern, queue)
        if c == '.' or c == x.c:
            if i == len(pattern) -1 and x.val is not None:
                queue.enqueue(prefix + str(x.c)) 
            if i < len(pattern) -1:
                self._collect_match(x.mid, prefix + x.c, i+1, pattern, queue)
        if c == '.' or c > x.c:
            self._collect_match(x.right, prefix, i, pattern, queue)

# * Unit tests the TST data type.
def test():
    st = TST()
    st.put("abc", 0)
    keys = [k for k in st.keys()]
    assert keys == ["abc"]
    assert st.get("abc") == 0
    st.put("a", 1)
    st.put("b", 2)
    st.put("c", 3)
    #st.delete("abc")
    #assert "abc" not in [k for k in st.keys()]
    assert st.get("a") == 1
    assert st.get("b") == 2
    assert st.get("c") == 3
    st.put("hello", 10)
    assert st.contains("hello")
    assert st.contains("not there") is False
    st.put("he", 20)
    assert st.longest_prefix_of("hell") == "he"
    st.put("jello", 30)
    q = st.keys_that_match(".e.l.")
    assert q.size() == 2 and "jello" in q and "hello" in q
    print("tests passed.")

if __name__ == '__main__':
    test()
    st = TST()
    i = 0
    # build symbol table from stdin
    print("Insert keys (Ctrl-D to stop):")
    while not stdio.isEmpty():
        key = stdio.readString()
        st.put(key, i)
        i += 1
    # print results
    if st.size() < 100:
        print('keys(""):')
        for key in st.keys():
            print("{} {}".format(key, st.get(key)))
        print()
            
        print("longestPrefixOf(\"shellsort\"):")
        print(st.longest_prefix_of("shellsort"))
        print()

        print("longestPrefixOf(\"shell\"):")
        print(st.longest_prefix_of("shell"))
        print()

        print("keysWithPrefix(\"shor\"):")
        for s in st.keys_with_prefix("shor"):
            print(s)
        print()

        print("keysThatMatch(\".he.l.\"):")
        for s in st.keys_that_match(".he.l."):
            print(s)



# Created for BADS 2018
# See README.md for details
# This is python3
import struct
import sys


"""
Binary standard output. This class provides methods for converting
some primitive type variables (boolean, byte, char, and int)
to sequences of bits and writing them
to an output stream.
The output stream can be standard output or another outputstream.
Uses big-endian (most-significant byte first).

The client must flush() the output stream when finished writing bits.

The client should not intermix calls to BinaryOut with calls to stdout;
otherwise unexpected behavior will result.
"""
class BinaryStdOut:
	out = sys.stdout.buffer
	buffer_ = 0
	n = 0
	is_init = False
	@staticmethod
	def _initialize():
		BinaryStdOut.buffer_ = 0 #8-bit buffer of bits to write out
		BinaryStdOut.n = 0 #number of bits used in buffer
		BinaryStdOut.is_init = True
	@staticmethod
	def _write_bit(x):
		if(not BinaryStdOut.is_init):
			BinaryStdOut._initialize()
		BinaryStdOut.buffer_ <<= 1
		if(x):
			BinaryStdOut.buffer_ |=1
		BinaryStdOut.n += 1
		if(BinaryStdOut.n == 8):
			BinaryStdOut._clear_buffer()
	@staticmethod
	def _write_byte(x):
		if(not BinaryStdOut.is_init):
			BinaryStdOut._initialize()
		assert x >= 0 and x < 256
		#optimized if byte-alligned
		if(BinaryStdOut.n == 0):
			BinaryStdOut.out.write(struct.pack('B',x))
			return
		#otherwise write one bit at a time
		for i in range(0,8):
			bit = ((x >> (8 - i - 1)) & 1) == 1
			BinaryStdOut._write_bit(bit)
	@staticmethod
	def _clear_buffer():
		if(not BinaryStdOut.is_init):
			BinaryStdOut._initialize()
		if(BinaryStdOut.n == 0):
			return
		if(BinaryStdOut.n > 0):
			BinaryStdOut.buffer_ <<= (8-BinaryStdOut.n)
		BinaryStdOut.out.write(struct.pack('B',BinaryStdOut.buffer_))
		BinaryStdOut.n = 0
		BinaryStdOut.buffer_ = 0
	@staticmethod
	def flush():
		BinaryStdOut._clear_buffer()
		BinaryStdOut.out.flush()
	@staticmethod
	def close():
		BinaryStdOut.flush()
		BinaryStdOut.out.close()
		BinaryStdOut.is_init = False
	@staticmethod
	def write_bool(x):
		BinaryStdOut._write_bit(x)
	@staticmethod
	def write_byte(x):
		BinaryStdOut._write_byte(x & 0xff)
	@staticmethod
	def write_int(x, r=32):
		if(r == 32):
			BinaryStdOut._write_byte(((x >> 24)& 0xff))
			BinaryStdOut._write_byte(((x >> 16)& 0xff))
			BinaryStdOut._write_byte(((x >> 8)& 0xff))
			BinaryStdOut._write_byte(((x >> 0)& 0xff))
			return
		if(r < 1 or r > 16):
			raise ValueError("Illegal value for r = {}".format(r))
		if(x < 0 or x >= (1 << r)):
			raise ValueError("Illegal {}-bit char = {}".format(r,x))
		for i in range(0,r):
			bit = ((x >> (r - i - 1)) & 1) == 1
			BinaryStdOut._write_bit(bit)
	@staticmethod
	def write_char(x, r=8):
		if(r==8):
			if(ord(x)<0 or ord(x)>=256):
				raise ValueError("Illegal 8-bit char = {}".format(x))
			BinaryStdOut._write_byte(ord(x))
			return
		if(r < 1 or r > 16):
			raise ValueError("Illegal value for r = {}".format(r))
		if(ord(x) >= (1 << r)):
			raise ValueError("Illegal {}-bit char = {}".format(r,x))
		for i in range(0,r):
			bit = ((x >> (r - i - 1)) & 1) == 1
			BinaryStdOut._write_bit(bit)

	def write_string(s, r=8):
		for i in s:
			BinaryStdOut.write_char(i,r)
def main():
	for i in sys.argv[1]:
		BinaryStdOut.write_char(i)
	BinaryStdOut.close()
if __name__ == '__main__':
	main()



# Created for BADS 2018
# See README.md for details
# Python 3

"""
The UF module implements several versions of the union-find data structure (also known as the
    disjoint-sets data type). It supports the union and find operations, along
    with a connected operation for determining whether two sites are in the same
    component and a count operation that returns the total number of components.

    The union-find data type models connectivity among a set of n sites, named 0
    through n-1. The is-connected-to relation must be an equivalence relation:
        * Reflexive: p is connected to p.
        * Symmetric: If p is connected to q, then q is connected to p.
        * Transitive: If p is connected to q and q is connected to r, then
                           p is connected to r.
"""
import sys
from typing import *

from itu.algs4.stdlib import stdio


class UF:
    """
    This is an implementation of the union-find data structure - see module documentation for
    more info.

    This implementation uses weighted quick union by rank with path compression by
    halving. Initializing a data structure with n sites takes linear time. Afterwards,
    the union, find, and connected operations take logarithmic time (in the worst case)
    and the count operation takes constant time. Moreover, the amortized time per union,
    find, and connected operation has inverse Ackermann complexity.

    For additional documentation, see Section 1.5 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """

    def __init__(self, n: int) -> None:
        """
        Initializes an empty union-find data structure with n sites,
        0 through n-1. Each site is initially in its own component.

        :param n: the number of sites
        """
        self._count = n
        self._parent = list(range(n))
        self._rank = [0]*n

    def _validate(self, p: int) -> None:
        # validate that p is a valid index
        n = len(self._parent)
        if p < 0 or p >= n:
            raise ValueError('index {} is not between 0 and {}'.format(p, n - 1))

    def union(self, p: int, q: int) -> None:
        """
        Merges the component containing site p with the
        component containing site q.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        """
        root_p = self.find(p)
        root_q = self.find(q)
        if root_p == root_q:
            return

        # make root of smaller rank point to root of larger rank
        if self._rank[root_p] < self._rank[root_q]:
            self._parent[root_p] = root_q
        elif self._rank[root_p] > self._rank[root_q]:
            self._parent[root_q] = root_p
        else:
            self._parent[root_q] = root_p
            self._rank[root_p] += 1

        self._count -= 1

    def find(self, p: int) -> int:
        """
        Returns the component identifier for the component containing site p.

        :param p: the integer representing one site
        :return: the component identifier for the component containing site p
        """
        self._validate(p)
        while p != self._parent[p]:
            self._parent[p] = self._parent[self._parent[p]] # path compression by halving
            p = self._parent[p]
        return p

    def connected(self, p: int, q: int) -> bool:
        """
        Returns true if the two sites are in the same component.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        :return: true if the two sites p and q are in the same component; false otherwise
        """
        return self.find(p) == self.find(q)

    def count(self) -> int:
        return self._count

class QuickUnionUF:
    """
    This is an implementation of the union-find data structure - see module documentation for
    more info.

    This implementation uses quick union. Initializing a data structure with n sites takes 
    linear time. Afterwards, the union, find, and connected operations take linear time 
    (in the worst case) and the count operation takes constant time. For alternate implementations 
    of the same API, see UF, QuickFindUF, and WeightedQuickUnionUF.

    For additional documentation, see Section 1.5 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """

    def __init__(self, n: int) -> None:
        """
        Initializes an empty union-find data structure with n sites,
        0 through n-1. Each site is initially in its own component.

        :param n: the number of sites
        """
        self._count = n
        self._parent = list(range(n))

    def _validate(self, p: int) -> None:
        # validate that p is a valid index
        n = len(self._parent)
        if p < 0 or p >= n:
            raise ValueError('index {} is not between 0 and {}'.format(p, n - 1))

    def union(self, p: int, q: int) -> None:
        """
        Merges the component containing site p with the
        component containing site q.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        """
        root_p = self.find(p)
        root_q = self.find(q)
        if root_p == root_q:
            return

        self._parent[root_p] = root_q

        self._count -= 1

    def find(self, p: int) -> int:
        """
        Returns the component identifier for the component containing site p.

        :param p: the integer representing one site
        :return: the component identifier for the component containing site p
        """
        self._validate(p)
        while p != self._parent[p]:
            p = self._parent[p]
        return p

    def connected(self, p: int, q: int) -> bool:
        """
        Returns true if the two sites are in the same component.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        :return: true if the two sites p and q are in the same component; false otherwise
        """
        return self.find(p) == self.find(q)

    def count(self) -> int:
        return self._count
    
class WeightedQuickUnionUF:
    """
    This is an implementation of the union-find data structure - see module documentation for
    more info.

    This implementation uses weighted quick union by size (without path compression). 
    Initializing a data structure with n sites takes linear time. Afterwards, the union, find, 
    and connected operations take logarithmic time (in the worst case) and the count operation 
    takes constant time. For alternate implementations of the same API, see UF, QuickFindUF, 
    and QuickUnionUF.

    For additional documentation, see Section 1.5 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """

    def __init__(self, n: int) -> None:
        """
        Initializes an empty union-find data structure with n sites,
        0 through n-1. Each site is initially in its own component.

        :param n: the number of sites
        """
        self._count = n
        self._parent = list(range(n))
        self._size = [1]*n

    def _validate(self, p: int) -> None:
        # validate that p is a valid index
        n = len(self._parent)
        if p < 0 or p >= n:
            raise ValueError('index {} is not between 0 and {}'.format(p, n - 1))

    def union(self, p: int, q: int) -> None:
        """
        Merges the component containing site p with the
        component containing site q.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        """
        root_p = self.find(p)
        root_q = self.find(q)
        if root_p == root_q:
            return

        # make root of smaller rank point to root of larger rank
        if self._size[root_p] < self._size[root_q]:
            small, large = root_p, root_q
        else:
            small, large = root_q, root_p
            
        self._parent[small] = large
        self._size[large] += self._size[small]
      
        self._count -= 1

    def find(self, p: int) -> int:
        """
        Returns the component identifier for the component containing site p.

        :param p: the integer representing one site
        :return: the component identifier for the component containing site p
        """
        self._validate(p)
        while p != self._parent[p]:
            p = self._parent[p]
        return p

    def connected(self, p: int, q: int) -> bool:
        """
        Returns true if the two sites are in the same component.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        :return: true if the two sites p and q are in the same component; false otherwise
        """
        return self.find(p) == self.find(q)

    def count(self) -> int:
        return self._count

class QuickFindUF:
    """
    This is an implementation of the union-find data structure - see module documentation for
    more info.

    This implementation uses quick find. Initializing a data structure with n sites takes linear time.
    Afterwards, the find, connected, and count operations take constant time but the union operation
    takes linear time.

    For additional documentation, see Section 1.5 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """

    def __init__(self, n: int) -> None:
        """
        Initializes an empty union-find data structure with n sites,
        0 through n-1. Each site is initially in its own component.

        :param n: the number of sites
        """
        self._count = n
        self._id = list(range(n))

    def _validate(self, p: int) -> None:
        # validate that p is a valid index
        n = len(self._id)
        if p < 0 or p >= n:
            raise ValueError('index {} is not between 0 and {}'.format(p, n - 1))

    def union(self, p: int, q: int) -> None:
        """
        Merges the component containing site p with the
        component containing site q.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        """
        self._validate(p)
        self._validate(q)

        p_id = self._id[p] # needed for correctness
        q_id = self._id[q] # to reduce the number of array accesses

        # p and q are already in the same component
        if p_id == q_id:
            return

        for i in range(len(self._id)):
            if self._id[i] == p_id:
                self._id[i] = q_id
        self._count -= 1

    def find(self, p: int) -> int:
        """
        Returns the component identifier for the component containing site p.

        :param p: the integer representing one site
        :return: the component identifier for the component containing site p
        """
        self._validate(p)
        return self._id[p]

    def connected(self, p: int, q: int) -> bool:
        """
        Returns true if the two sites are in the same component.

        :param p: the integer representing one site
        :param q: the integer representing the other site
        :return: true if the two sites p and q are in the same component; false otherwise
        """
        self._validate(p)
        self._validate(q)
        return self._id[p] == self._id[q]

    def count(self):
        return self._count


# Reads in a an integer n and a sequence of pairs of integers
# (between 0 and n-1) from standard input or a file
# supplied as argument to the program, where each integer
# in the pair represents some site; if the sites are in different
# components, merge the two components and print the pair to standard output.
if __name__ == '__main__':
    if len(sys.argv) > 1:
        try:
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")
    n = stdio.readInt()
    uf = UF(n)
    while not stdio.isEmpty():
        p = stdio.readInt()
        q = stdio.readInt()
        if uf.connected(p, q):
            continue
        uf.union(p, q)
        print('{} {}'.format(p, q))
    print('number of components: {}'.format(uf.count()))



# Created for BADS 2018
# See README.md for details
# Python 3

"""
This module provides functions for sorting arrays of strings using msd sort.

For additional documentation, see Section 5.1 of Algorithms, 4th Edition 
by Robert Sedgewick and Kevin Wayne.
"""

cutoff = 15

# compare characters at position d
def _less(a, b, d):
    return a[d] < b[d]

# insertion sort a[lo..hi], starting at dth character
def _insertion(a, lo, hi, d):
    for i in range(lo, hi+1):
        j = i
        while j>lo and _less(a[j], a[j-1], d):
            a[j], a[j-1] = a[j-1], a[j]
            j -= 1

# sort from a[lo] to a[hi], starting at the dth character
def _sort(a, lo, hi, d, aux, radix):
    global cutoff
    
    if hi <= lo+cutoff:
        _insertion(a, lo, hi, d)
        return
    
    # compute frequency counts
    count = [0]*(radix+2)
    for i in range(hi+1):
        ch = ord(a[i][d]) # get number representation of character
        count[ch + 2] += 1

    # compute cumulates
    for r in range(radix+1):
        count[r+1] += count[r]

    # move data
    for i in range(hi+1):
        ch = ord(a[i][d])
        aux[count[ch+1]] = a[i]
        count[ch+1] += 1

    # copy back
    for i in range(hi+1):
        a[i] = aux[i-lo]
    
    # recursively sort for each character (excludes sentinel -1)
    for r in range(radix):
        _sort(a, lo + count[r], lo + count[r+1] - 1, d+1, aux, radix)

def sort(a, radix=256):
    """
    Rearranges the array of 32-bit integers in ascending order.
    Currently assumes that the integers are nonnegative.
    
    :param a: the array to be sorted
    """
    n = len(a)
    aux = [None]*n
    _sort(a, 0, n-1, 0, aux, radix)
        

import sys

from itu.algs4.stdlib import stdio

if __name__ == '__main__':
    if len(sys.argv) > 1:
        try: 
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")
    
    a = stdio.readAllStrings()
    sort(a)
    for elem in a:
        print(elem)    



# Created for BADS 2018
# see README.md for details
# This is python3 

class CC:
    """
    The CC class represents a data type for 
    determining the connected components in an undirected graph.
    The id operation determines in which connected component
    a given vertex lies the connected operation
    determines whether two vertices are in the same connected component
    the count operation determines the number of connected
    components and the size operation determines the number
    of vertices in the connect component containing a given vertex.

    The component identifier of a connected component is one of the
    vertices in the connected component: two vertices have the same component
    identifier if and only if they are in the same connected component.

    This implementation uses depth-first search.
    The constructor takes time proportional to V + E
    (in the worst case),
    where V is the number of vertices and E is the number of edges.
    Afterwards, the id, count, connected,
    and size operations take constant time.
    """

    def __init__(self, G):
        """
        Computes the connected components of the undirected graph G.

        :param G: the undirected graph
        """
        self._marked = [False] * G.V()  # marked[v] = has vertex v been marked?
        self._id = [None] * G.V()       # id[v] = id of connected component containing v
        self._size = [0] * G.V()        # size[id] = number of vertices in given component
        self._count = 0                 # number of connected components

        for v in range(G.V()):
            if not self._marked[v]:
                self._dfs(G, v) 
                self._count += 1

    def _dfs(self, G, v):
        # depth-first search for a Graph
        self._marked[v] = True
        self._id[v] = self._count
        self._size[self._count] += 1
        for w in G.adj(v):
            if not self._marked[w]:
                self._dfs(G, w)
    
    def id(self, v):
        """
        Returns the component id of the connected component containing vertex v.
     
        :param v: the vertex
        :returns: the component id of the connected component containing vertex v
        :raises ValueError: unless 0 <= v < V
        """    
        self._validate_vertex(v)
        return self._id[v]

    def size(self, v):
        """
        Returns the number of vertices in the connected component containing vertex v.
     
        :param v: the vertex
        :returns: the number of vertices in the connected component containing vertex v
        :raises ValueError: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._size[self._id[v]]

    def count(self):
        """
        Returns the number of connected components in the graph G.

        :returns: the number of connected components in the graph G
        """
        return self._count

    def connected(self, v, w):
        """
        Returns true if vertices v and w are in the same connected component.
     
        :param v: one vertex
        :param w: the other vertex
        :returns: True if vertices v and w are in the same connected component; 
                    False otherwise
        :raises ValueError: unless 0 <= v < V
        :raises ValueError: unless 0 <= w < V
        """
        self._validate_vertex(v)
        self._validate_vertex(w)
        return self.id(v) == self.id(w)

    def _validate_vertex(self, v):
        # Raises a ValueError n unless 0 <= v < V
        V = len(self._marked)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))


class CCBook:
    def __init__(self, G):
        self._marked = [False] * G.V()  # marked[v] = has vertex v been marked?
        self._id = [None] * G.V()       # id[v] = id of connected component containing v
        self._count = 0                 # number of connected components

        for s in range(G.V()):
            if not self._marked[s]:
                self._dfs(G, s) 
                self._count += 1

    def _dfs(self, G, v):
        self._marked[v] = True
        self._id[v] = self._count
        for w in G.adj(v):
            if not self._marked[w]:
                self._dfs(G, w)
    
    def connected(self, v, w):
        return self._id[v] == self._id[w]

    def id(self, v):
        return self._id[v]

    def count(self):
        return self._count    

if __name__ == "__main__":
    import sys
    from itu.algs4.fundamentals.queue import Queue
    from itu.algs4.stdlib.instream import InStream    
    from itu.algs4.stdlib import stdio    
    from itu.algs4.graphs.graph import Graph    

    In = InStream(sys.argv[1])
    G = Graph.from_stream(In)
    cc = CC(G)

    # number of connected components
    m = cc.count()
    stdio.writef("%i components\n", m)

    # compute list of vertices in each connected component
    components = [Queue() for _ in range(m)]
    for v in range(G.V()):
        components[cc.id(v)].enqueue(v)

    # print results
    for i in range(m):
        for v in components[i]:
            stdio.writef("%i ", v)
        stdio.writeln()



from itu.algs4.fundamentals import binary_search


class ThreeSumFast:
    @staticmethod
    def count(a):
        # Count triples that sum to 0
        a.sort()
        n = len(a)
        count = 0
        for i in range(n):
            for j in range(i+1, n):
                if binary_search.index_of(a, -a[i]-a[j]) > j:            
                    count += 1
        return count



# Created for BADS 2018
# see README.md for details
# Python 3

import sys
from abc import abstractmethod
from typing import Generic, List, Optional, Sequence, TypeVar

from typing_extensions import Protocol

from ..errors.errors import IllegalArgumentException, NoSuchElementException
from ..fundamentals.queue import Queue

sys.setrecursionlimit(10**5)

"""
The BST class represents an ordered symbol table of generic
key-value pairs.

This implementation uses an unbalanced, binary search tree.

For additional details and documentation, see Section 3.2 of Algorithms,
4th Edition by Robert Sedgewick and Kevin Wayne.

:original author: Robert Sedgewick and Kevin Wayne
:original java code: https://algs4.cs.princeton.edu/32bst/BST.java.html

"""


Val = TypeVar('Val')
Key = TypeVar('Key', bound = 'Comparable')

class Comparable(Protocol):
    @abstractmethod
    def __lt__(self: Key, other: Key) -> bool:
        pass

class Node(Generic[Key, Val]):
    def __init__(self, key: Key, value: Optional[Val], size: int):
        self.left:  Optional[Node[Key,Val]] = None      # root of left subtree
        self.right: Optional[Node[Key,Val]] = None      # root of right subtree
        self.key: Key = key         # sorted by key
        self.value: Optional[Val] = value     # associated data
        self.size: int = size       # number of nodes in subtree

class BST(Generic[Key, Val]):
    def __init__(self) -> None:
        """
        Initialises empty symbol table
        """
        self._root: Optional[Node[Key, Val]] = None           # root of BST

    def is_empty(self) -> bool:
        """
        Returns true if this symbol table is empty
        """
        return self.size() == 0

    def contains(self, key: Key) -> bool:
        """
        Does this symbol table contain the given key?
        :param key: the key to search for
        :return boolean: true if symbol table contains key, false otherwise
        """
        return self.get(key) != None

    def size(self) -> int:
        """
        Returns the number of key-value pairs in this symbol table
        """
        return self._size(self._root)
        
    def __len__(self) -> int:
        return self.size()

    def _size(self, node: Optional[Node[Key, Val]]) -> int:
        """
        Returns the number of key-value pairs in BST rooted at node

        :param node: The node which act as root
        """
        if node is None:
            return 0
        else:
            return node.size

    def get(self, key: Key) -> Optional[Val]:
        """
        Returns the value associated with the given key

        :param key: The key whose value is returned
        :return: the value associated with the given key if the key
        is in the symbol table, None otherwise
        """
        return self._get(self._root, key)

    def _get(self, node: Optional[Node[Key, Val]], key: Key) -> Optional[Val]:
        if node is None:
            return None
        else:
            if key < node.key:
                return self._get(node.left, key)
            elif key > node.key:
                return self._get(node.right, key)
            else:
                return node.value

    def put(self, key: Key, value: Optional[Val]) -> None:
        """
        Inserts the specified key-value pair into the symbol table,
        overwriting the old value with the new value if the symbol table
        already contains the specified key. Deletes the specified key (and
        its associated value) from this symbol table if the specified value
        is None.

        :param key, value: the key-value pair to be inserted
        """
        if value is None:
            self.delete(key)
            return
        self._root = self._put(self._root, key, value)

    def _put(self, node: Optional[Node[Key,Val]], key: Key, value: Optional[Val]) -> Node[Key, Val]:
        if node is None:
            newnode: Node[Key, Val] = Node(key, value, 1)
            return newnode
        else:
            if key < (node.key):
                node.left = self._put(node.left, key, value)
            elif key > (node.key):
                node.right = self._put(node.right, key, value)
            else:
                node.value = value
            node.size = 1 + self._size(node.left) + self._size(node.right)
            return node

    def delete_min(self) -> None:
        """
        Removes the smallest key and associated value from the symbol table
        TODO exception?
        """
        if self.is_empty():
            raise NoSuchElementException("calls min() with empty symbol table")
        else:
            assert self._root is not None
            self._root = self._delete_min(self._root)

    def _delete_min(self, node: Node[Key, Val]) -> Optional[Node[Key, Val]]:
        if node.left is None:
            return node.right
        else:
            node.left = self._delete_min(node.left)
            node.size = self._size(node.left) + self._size(node.right) + 1
            return node

    def delete_max(self) -> None:
        """
        Removes the largest key and associated value from the symbol table
        """
        if self.is_empty():
            raise NoSuchElementException("calls max() with empty symbol table")
        else:
            assert self._root is not None
            self._root = self._delete_max(self._root)

    def _delete_max(self, node: Node[Key, Val]) -> Optional[Node[Key, Val]]:
        if node.right is None:
            return node.left
        else:
            node.right = self._delete_max(node.right)
        node.size = self._size(node.left) + self._size(node.right) + 1
        return node


    def delete(self, key: Key) -> None:
        """
        Removes the specified key and its associated value from this symbol table
        (if the key is in this symbol table)
        """
        self._root = self._delete(self._root, key)

    def _delete(self, node: Optional[Node[Key,Val]], key: Key) -> Optional[Node[Key,Val]]:
        if node is None:
            return None
        else:
            if key.__lt__(node.key):
                node.left = self._delete(node.left, key)
            elif node.key < key:
                node.right = self._delete(node.right, key)
            else:
                if node.right is None:
                    return node.left
                elif node.left is None:
                    return node.right
                else:
                    temp_node = node
                    assert temp_node.right is not None
                    node = self._min(temp_node.right)
                    node.right = self._delete_min(temp_node.right)
                    node.left = temp_node.left

            node.size = self._size(node.left) + self._size(node.right) + 1
            return node

    def min(self) -> Key:
        """
        Returns the smallest key in the BST
        """
        if self.is_empty():
            raise NoSuchElementException("calls min() with empty symbol table")
        else:
            assert self._root is not None
            return self._min(self._root).key

    def _min(self, node: Node[Key, Val]) -> Node[Key, Val]:
        if node.left is None:
            return node
        else:
            return self._min(node.left)

    def max(self) -> Key:
        """
        Returns the larget key in the symbol table
        """
        if self.is_empty():
            raise NoSuchElementException("calls max() with empty symbol table")
        else:
            assert self._root is not None
            return self._max(self._root).key

    def _max(self, node: Node[Key, Val]) -> Node[Key, Val]:
        if node.right is None:
            return node
        else:
            return self._max(node.right)

    def floor(self, key: Key) -> Key:
        """
        Returns the largest key in the symbol table less than or equal to key
        Raises NoSuchElementException if no such key exists.
        """
        if self.is_empty():
            raise NoSuchElementException("calls floor() with empty symbol table")

        node = self._floor(self._root, key)
        if node is None:
            raise NoSuchElementException("calls floor() with key < min")
        else:
            return node.key

    def _floor(self, node: Optional[Node[Key,Val]], key: Key) -> Optional[Node[Key, Val]]:
        if node is None:
            return None
        elif key == node.key:
            return node
        elif key < node.key:
            return self._floor(node.left, key)
        temp_node = self._floor(node.right, key)
        if temp_node is not None:
            return temp_node
        return node

    def ceiling(self, key: Key) -> Key:
        """
        Returns the smallest key in the symbol table greater than or equal to key
        Raises NoSuchElementException if no such key exists.
        """
        if self.is_empty():
            raise NoSuchElementException("calls ceiling() with empty symbol table")

        node = self._ceiling(self._root, key)
        if node is None:
            raise NoSuchElementException("calls ceiling() with key > max")
        else:
            return node.key

    def _ceiling(self, node: Optional[Node[Key,Val]], key: Key) -> Optional[Node[Key, Val]]:
        if node is None:
            return None
        elif key == node.key:
            return node
        elif key > node.key:
            return self._ceiling(node.right, key)
        temp_node = self._ceiling(node.left, key)
        if temp_node is not None:
            return temp_node
        return node

    def keys(self) -> Queue[Key]:
        """
        Returns all keys in the symbol table as a list.
        """
        if self.is_empty():
            return Queue()
        return self.range_keys(self.min(), self.max())

    def range_keys(self, lo: Key, hi: Key) -> Queue[Key]:
        """
        returns all keys in the symbol table in the given range as a list

        :param lo: minimum endpoint
        :param hi: maximum endpoint
        :return: all keys in symbol table between lo (inclusive) and hi (inclusive)
        """
        queue: Queue[Key] = Queue()
        self._range_keys(self._root, queue, lo, hi)
        return queue

    def _range_keys(self, node: Optional[Node[Key, Val]], queue: Queue[Key], lo: Key, hi: Key) -> None:
        if node is None:
            return
        elif lo < node.key:
            self._range_keys(node.left, queue, lo, hi)
        if not lo > node.key and not hi < node.key:
            queue.enqueue(node.key)
        if hi > node.key:
            self._range_keys(node.right, queue, lo, hi)
            
    def select(self, k: int) -> Key:
        """
        Return the kth smallest key in the symbol table.
        :param k: the order statistic
        :return: the kth smallest key in the symbol table
        :raises IllegalArgumentException: unless k is between 0 and n-1
        """
        if k < 0 or k >= self.size():
            raise IllegalArgumentException("argument to select() is invalid: {}".format(k))
        assert self._root is not None
        x = self._select(self._root, k)
        return x.key

    def _select(self, x: Node[Key, Val], k: int) -> Node[Key, Val]:
        """
        Returns the node with key of rank k in the subtree rooted at x
        :rtype: Node
        """
        t = self._size(x.left)
        if t > k:
            assert x.left is not None
            return self._select(x.left, k)
        elif t < k:
            assert x.right is not None
            return self._select(x.right, k-t-1)
        else:
            return x

    def rank(self, key: Key) -> int:
        """
        Returns the number of keys in the symbol table strictly less than key.
        :param key: the key
        :return: the number of keys in the symbol table strictly less than key
        :rtype: int
        :raises IllegalArgumentException: if key is None
        """
        if key is None:
            raise IllegalArgumentException("argument to rank() is None")
        return self._rank(key, self._root)

    def _rank(self, key: Key, x: Optional[Node[Key, Val]]) -> int:
        """
        Returns the number of keys less than key in the subtree rooted at x.
        :rtype: int
        """
        if x is None:
            return 0
        if key < x.key:
            return self._rank(key, x.left)
        if key > x.key:
            return 1 + self._size(x.left) + self._rank(key, x.right)
        else:
            return self._size(x.left)

    def size_range(self, lo: Key, hi: Key) -> int:
        """
        Returns the number of keys in the symbol table in the given range.
        :param lo: minimum endpoint
        :param hi: maximum endpoint
        :return: the number of keys in the symbol table between lo
        (inclusive) and hi (inclusive)
        :rtype: int
        :raises IllegalArgumentException: if either lo or hi is None
        """
        if lo is None:
            return IllegalArgumentException("first argument to size() is None")
        if hi is None:
            return IllegalArgumentException("second argument to size() is None")
        if lo > hi:
            return 0
        if self.contains(hi):
            return self.rank(hi) - self.rank(lo) + 1
        else:
            return self.rank(hi) - self.rank(lo)

    def height(self) -> int:
        """
        Returns the height of the BST (for debugging)
        """
        return self._height(self._root)

    def _height(self, node: Optional[Node[Key, Val]]) -> int:
        if node is None:
            return -1
        else:
            return 1 + max(self._height(node.left), self._height(node.right))

    def level_order(self) -> Queue[Key]:
        """
        Returns the keys in the BST in level order (for debugging)
        """
        queue: Queue[Optional[Node[Key, Val]]] = Queue()
        keys: Queue[Key] =  Queue()
        queue.enqueue(self._root) 
        while len(queue) > 0:
            node = queue.dequeue()
            if node is None:
                continue
            else:
                keys.enqueue(node.key)
                queue.enqueue(node.left)
                queue.enqueue(node.right)
        return keys



# Created for BADS 2018
# See README.md for details
# This is python3 
import sys


class BoyerMoore:
    """
    The BoyerMoore class finds the first occurence of a pattern string
    in a text string.
    This implementation uses the Boyer-Moore algorithm (with the bad-character
    rule, but not the strong good suffix rule).
    """
    
    def __init__(self, pat):
        """
        Preprocesses the pattern string.
        :param pat: the pattern string
        """
        self.pat = pat
        M = len(pat)
        R = 256
        self.right = [-1 for i in range(0,R)] #-1 for chars not in pattern
        for j in range(0,M):
            self.right[ord(pat[j])] = j
            
    def search(self, txt):
        """
        Returns the index of the first occurrrence of the pattern string
        in the text string.
        :param txt: the text string
        :return: the index of the first occurrence of the pattern string 
        in the text string; N if no such match
        """
        N = len(txt)
        M = len(self.pat)
        skip = 0
        i = 0
        #for i in range(0,N-M+1,skip):
        while(i <= N-M):
            skip = 0
            for j in range(M-1,-1,-1):
                if not(self.pat[j] == txt[i+j]):
                    skip = j-self.right[ord(txt[i+j])]
                    if(skip < 1):
                        skip = 1
                    break
            if(skip == 0):
                return i
            i += skip
        return N
    
def main():
    """
    Takes a pattern string and an input string as command-line arguments;
    searches for the pattern string in the text string; and prints
    the first occurrence of the pattern string in the text string.
    Will print the pattern after the end of the string if no match is found.
    """
    pat = sys.argv[1]
    txt = sys.argv[2]
    bm = BoyerMoore(pat)
    print("text:    {}".format(txt))
    offset = bm.search(txt)
    print("pattern:",end=" ")
    for i in range(0,offset):
        print("", end=" ")
    print(pat)
    
if __name__ == "__main__":
    main()



from itu.algs4.fundamentals import binary_search


class TwoSumFast:
    @staticmethod
    def count(a):
        # Count pairs that sum to 0
        a = sorted(a)
        n = len(a)
        count = 0
        for i in range(n):
            if binary_search.index_of(a, -a[i]) > i:            
                count += 1
        return count



import sys

from itu.algs4.fundamentals.queue import Queue
from itu.algs4.fundamentals.uf import WeightedQuickUnionUF
from itu.algs4.graphs.edge import Edge
from itu.algs4.graphs.edge_weighted_graph import EdgeWeightedGraph
from itu.algs4.sorting.min_pq import MinPQ
from itu.algs4.stdlib import stdio
from itu.algs4.stdlib.instream import InStream

# Created for BADS 2018
# See README.md for details
# Python 3


class KruskalMST:
    """
    The KruskalMST class represents a data type for computing a
    minimum spanning tree in an edge-weighted graph.
    The edge weights can be positive, zero, or negative and need not
    be distinct. If the graph is not connected, it computes a minimum
    spanning forest, which is the union of minimum spanning trees
    in each connected component. The weight method returns the
    weight of a minimum spanning tree and the edges method
    returns its edges.
    This implementation uses Kruskal's algorithm and the
    union-find data type.
    The constructor takes time proportional to E log E
    and extra space (not including the graph) proportional to V,
    where V is the number of vertices and E is the number of edges-
    Afterwards, the weight method takes constant time
    and the edges method takes time proportional to V.
    """
    def __init__(self, G):
        """
        Computes a minimum spanning tree (or forest) of an edge-weighted graph.
        :param G: the edge-weighted graph
        """
        self._weight = 0
        self._mst = Queue()
        pq = MinPQ()

        for e in G.edges():
            pq.insert(e)

        uf = WeightedQuickUnionUF(G.V())
        while not pq.is_empty() and self._mst.size() < G.V() - 1:
            e = pq.del_min()
            v = e.either()
            w = e.other(v)
            if not uf.connected(v, w):
                uf.union(v, w)
                self._mst.enqueue(e)
                self._weight += e.weight()

    def edges(self):
        """
        Returns the edges in a minimum spanning tree (or forest).
        :return: the edges in a minimum spanning tree (or forest)
        """
        return self._mst

    def weight(self):
        """
        Returns the sum of the edge weights in a minimum spanning tree (or forest).
        :return: the sum of the edge weights in a minimum spanning tree (or forest)
        """
        return self._weight


def main():
    """
    Creates an edge-weighted graph from an input file, runs Kruskal's algorithm on it,
    and prints the edges of the MST and the sum of the edge weights.
    """
    if len(sys.argv) > 1:
        stream = InStream(sys.argv[1])
        G = EdgeWeightedGraph.from_stream(stream)
        mst = KruskalMST(G)
        for e in mst.edges():
            print(e)
        print("{:.5f}".format(mst.weight()))


if __name__ == '__main__':
    main()



# Created for BADS 2018
# see README.md for details
# This is python3 
 
class DepthFirstSearch:
    """
    The DepthFirstSearch class represents a data type for 
    determining the vertices connected to a given source vertex s
    in an undirected graph. For versions that find the paths, see
    DepthFirstPaths and BreadthFirstPaths.

    This implementation uses depth-first search.
    The constructor takes time proportional to V + E
    (in the worst case),
    where V is the number of vertices and E is the number of edges.
    It uses extra space (not including the graph) proportional to V.
    """
    def __init__(self, G, s):
        """
        Computes the vertices in graph G that are
        connected to the source vertex s.
        :param G: the graph
        :param s: the source vertex
        :throws ValueError: unless 0 <= s < V
        """
        self._marked = [False] * G.V()  # marked[v] = is there an s-v path?        
        self._count = 0                 # number of vertices connected to s
        self._validateVertex(s)
        self._dfs(G, s)
    
    def _dfs(self, G, v):
        # depth first search from v        
        self._marked[v] = True
        self._count += 1
        for w in G.adj(v):
            if not self._marked[w]:
                self._dfs(G, w)

    def marked(self, v):
        """
        Is there a path between the source vertex s and vertex v?
     
        :param v: the vertex
        :returns: true if there is a path, false otherwise
        :raises ValueError: unless 0 <= v < V
        """
        self._validateVertex(v)
        return self._marked[v]

    def count(self):
        """
        Returns the number of vertices connected to the source vertex s.

        :returns: the number of vertices connected to the source vertex s
        """
        return self._count
    
    def _validateVertex(self, v):
        # throw an ValueError unless 0 <= v < V
        V = len(self._marked)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))

    
if __name__ == "__main__":
    from itu.algs4.graphs.graph import Graph
    from itu.algs4.stdlib.instream import InStream
    from itu.algs4.stdlib import stdio
    import sys

    In = InStream(sys.argv[1])
    G = Graph.from_stream(In)
    s = int(sys.argv[2])
    search = DepthFirstSearch(G, s)
    for v in range(G.V()):
        if search.marked(v):
            stdio.writef("%i ", v)
    stdio.writeln()

    if search.count() != G.V(): stdio.writeln("NOT connected")
    else:                       stdio.writeln("connected")



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
stddraw.py

The stddraw module defines functions that allow the user to create a
drawing.  A drawing appears on the canvas.  The canvas appears
in the window.  As a convenience, the module also imports the
commonly used Color objects defined in the color module.
"""

import os
import string
import sys
import time

import pygame
import pygame.font
import pygame.gfxdraw
from itu.algs4.stdlib import color
from itu.algs4.stdlib.color import (
    BLACK,
    BLUE,
    BOOK_BLUE,
    BOOK_LIGHT_BLUE,
    BOOK_RED,
    CYAN,
    DARK_BLUE,
    DARK_GRAY,
    DARK_GREEN,
    DARK_RED,
    GRAY,
    GREEN,
    LIGHT_GRAY,
    MAGENTA,
    ORANGE,
    PINK,
    RED,
    VIOLET,
    WHITE,
    YELLOW,
)

if (sys.hexversion < 0x03000000):
    import Tkinter
    import tkMessageBox
    import tkFileDialog
else:
    import tkinter as Tkinter
    import tkinter.messagebox as tkMessageBox
    import tkinter.filedialog as tkFileDialog

#-----------------------------------------------------------------------

# Define colors so clients need not import the color module.


#-----------------------------------------------------------------------

# Default Sizes and Values

_BORDER = 0.0
#_BORDER = 0.05
_DEFAULT_XMIN = 0.0
_DEFAULT_XMAX = 1.0
_DEFAULT_YMIN = 0.0
_DEFAULT_YMAX = 1.0
_DEFAULT_CANVAS_SIZE = 512
_DEFAULT_PEN_RADIUS = .005  # Maybe change this to 0.0 in the future.
_DEFAULT_PEN_COLOR = BLACK

_DEFAULT_FONT_FAMILY = 'Helvetica'
_DEFAULT_FONT_SIZE = 12

_xmin = None
_ymin = None
_xmax = None
_ymax = None

_fontFamily = _DEFAULT_FONT_FAMILY
_fontSize = _DEFAULT_FONT_SIZE

_canvasWidth = float(_DEFAULT_CANVAS_SIZE)
_canvasHeight = float(_DEFAULT_CANVAS_SIZE)
_penRadius = None
_penColor = _DEFAULT_PEN_COLOR
_keysTyped = []

# Has the window been created?
_windowCreated = False

#-----------------------------------------------------------------------
# Begin added by Alan J. Broder
#-----------------------------------------------------------------------

# Keep track of mouse status

# Has the mouse been left-clicked since the last time we checked?
_mousePressed = False

# The position of the mouse as of the most recent mouse click
_mousePos = None

#-----------------------------------------------------------------------
# End added by Alan J. Broder
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------

def _pygameColor(c):
    """
    Convert c, an object of type color.Color, to an equivalent object
    of type pygame.Color.  Return the result.
    """
    r = c.getRed()
    g = c.getGreen()
    b = c.getBlue()
    return pygame.Color(r, g, b)

#-----------------------------------------------------------------------

# Private functions to scale and factor X and Y values.

def _scaleX(x):
    return _canvasWidth * (x - _xmin) / (_xmax - _xmin)

def _scaleY(y):
    return _canvasHeight * (_ymax - y) / (_ymax - _ymin)

def _factorX(w):
    return w * _canvasWidth / abs(_xmax - _xmin)

def _factorY(h):
    return h * _canvasHeight / abs(_ymax - _ymin)

#-----------------------------------------------------------------------
# Begin added by Alan J. Broder
#-----------------------------------------------------------------------

def _userX(x):
    return _xmin + x * (_xmax - _xmin) / _canvasWidth

def _userY(y):
    return _ymax - y * (_ymax - _ymin) / _canvasHeight

#-----------------------------------------------------------------------
# End added by Alan J. Broder
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------

def setCanvasSize(w=_DEFAULT_CANVAS_SIZE, h=_DEFAULT_CANVAS_SIZE):
    """
    Set the size of the canvas to w pixels wide and h pixels high.
    Calling this function is optional. If you call it, you must do
    so before calling any drawing function.
    """
    global _background
    global _surface
    global _canvasWidth
    global _canvasHeight
    global _windowCreated

    if _windowCreated:
        raise Exception('The stddraw window already was created')

    if (w < 1) or (h < 1):
        raise Exception('width and height must be positive')

    _canvasWidth = w
    _canvasHeight = h
    _background = pygame.display.set_mode([w, h])
    pygame.display.set_caption('stddraw window (r-click to save)')
    _surface = pygame.Surface((w, h))
    _surface.fill(_pygameColor(WHITE))
    _windowCreated = True

def setXscale(min=_DEFAULT_XMIN, max=_DEFAULT_XMAX):
    """
    Set the x-scale of the canvas such that the minimum x value
    is min and the maximum x value is max.
    """
    global _xmin
    global _xmax
    min = float(min)
    max = float(max)
    if min >= max:
        raise Exception('min must be less than max')
    size = max - min
    _xmin = min - _BORDER * size
    _xmax = max + _BORDER * size

def setYscale(min=_DEFAULT_YMIN, max=_DEFAULT_YMAX):
    """
    Set the y-scale of the canvas such that the minimum y value
    is min and the maximum y value is max.
    """
    global _ymin
    global _ymax
    min = float(min)
    max = float(max)
    if min >= max:
        raise Exception('min must be less than max')
    size = max - min
    _ymin = min - _BORDER * size
    _ymax = max + _BORDER * size

def setPenRadius(r=_DEFAULT_PEN_RADIUS):
    """
    Set the pen radius to r, thus affecting the subsequent drawing
    of points and lines. If r is 0.0, then points will be drawn with
    the minimum possible radius and lines with the minimum possible
    width.
    """
    global _penRadius
    r = float(r)
    if r < 0.0:
        raise Exception('Argument to setPenRadius() must be non-neg')
    _penRadius = r * float(_DEFAULT_CANVAS_SIZE)

def setPenColor(c=_DEFAULT_PEN_COLOR):
    """
    Set the pen color to c, where c is an object of class color.Color.
    c defaults to stddraw.BLACK.
    """
    global _penColor
    _penColor = c

def setFontFamily(f=_DEFAULT_FONT_FAMILY):
    """
    Set the font family to f (e.g. 'Helvetica' or 'Courier').
    """
    global _fontFamily
    _fontFamily = f

def setFontSize(s=_DEFAULT_FONT_SIZE):
    """
    Set the font size to s (e.g. 12 or 16).
    """
    global _fontSize
    _fontSize = s

#-----------------------------------------------------------------------

def _makeSureWindowCreated():
    global _windowCreated
    if not _windowCreated:
        setCanvasSize()
        _windowCreated = True

#-----------------------------------------------------------------------

# Functions to draw shapes, text, and images on the background canvas.

def _pixel(x, y):
    """
    Draw on the background canvas a pixel at (x, y).
    """
    _makeSureWindowCreated()
    xs = _scaleX(x)
    xy = _scaleY(y)
    pygame.gfxdraw.pixel(
        _surface,
        int(round(xs)),
        int(round(xy)),
        _pygameColor(_penColor))

def point(x, y):
    """
    Draw on the background canvas a point at (x, y).
    """
    _makeSureWindowCreated()
    x = float(x)
    y = float(y)
    # If the radius is too small, then simply draw a pixel.
    if _penRadius <= 1.0:
        _pixel(x, y)
    else:
        xs = _scaleX(x)
        ys = _scaleY(y)
        pygame.draw.ellipse(
            _surface,
            _pygameColor(_penColor),
            pygame.Rect(
                xs-_penRadius,
                ys-_penRadius,
                _penRadius*2.0,
                _penRadius*2.0),
            0)

def _thickLine(x0, y0, x1, y1, r):
    """
    Draw on the background canvas a line from (x0, y0) to (x1, y1).
    Draw the line with a pen whose radius is r.
    """
    xs0 = _scaleX(x0)
    ys0 = _scaleY(y0)
    xs1 = _scaleX(x1)
    ys1 = _scaleY(y1)
    if (abs(xs0-xs1) < 1.0) and (abs(ys0-ys1) < 1.0):
        filledCircle(x0, y0, r)
        return
    xMid = (x0+x1)/2
    yMid = (y0+y1)/2
    _thickLine(x0, y0, xMid, yMid, r)
    _thickLine(xMid, yMid, x1, y1, r)

def line(x0, y0, x1, y1):
    """
    Draw on the background canvas a line from (x0, y0) to (x1, y1).
    """

    THICK_LINE_CUTOFF = 3 # pixels

    _makeSureWindowCreated()

    x0 = float(x0)
    y0 = float(y0)
    x1 = float(x1)
    y1 = float(y1)

    lineWidth = _penRadius * 2.0
    if lineWidth == 0.0: lineWidth = 1.0
    if lineWidth < THICK_LINE_CUTOFF:
        x0s = _scaleX(x0)
        y0s = _scaleY(y0)
        x1s = _scaleX(x1)
        y1s = _scaleY(y1)
        pygame.draw.line(
            _surface,
            _pygameColor(_penColor),
            (x0s, y0s),
            (x1s, y1s),
            int(round(lineWidth)))
    else:
        _thickLine(x0, y0, x1, y1, _penRadius/_DEFAULT_CANVAS_SIZE)

def circle(x, y, r):
    """
    Draw on the background canvas a circle of radius r centered on
    (x, y).
    """
    _makeSureWindowCreated()
    x = float(x)
    y = float(y)
    r = float(r)
    ws = _factorX(2.0*r)
    hs = _factorY(2.0*r)
    # If the radius is too small, then simply draw a pixel.
    if (ws <= 1.0) and (hs <= 1.0):
        _pixel(x, y)
    else:
        xs = _scaleX(x)
        ys = _scaleY(y)
        pygame.draw.ellipse(
            _surface,
            _pygameColor(_penColor),
            pygame.Rect(xs-ws/2.0, ys-hs/2.0, ws, hs),
            int(round(_penRadius)))

def filledCircle(x, y, r):
    """
    Draw on the background canvas a filled circle of radius r
    centered on (x, y).
    """
    _makeSureWindowCreated()
    x = float(x)
    y = float(y)
    r = float(r)
    ws = _factorX(2.0*r)
    hs = _factorY(2.0*r)
    # If the radius is too small, then simply draw a pixel.
    if (ws <= 1.0) and (hs <= 1.0):
        _pixel(x, y)
    else:
        xs = _scaleX(x)
        ys = _scaleY(y)
        pygame.draw.ellipse(
            _surface,
            _pygameColor(_penColor),
            pygame.Rect(xs-ws/2.0, ys-hs/2.0, ws, hs),
            0)

def rectangle(x, y, w, h):
    """
    Draw on the background canvas a rectangle of width w and height h
    whose lower left point is (x, y).
    """
    global _surface
    _makeSureWindowCreated()
    x = float(x)
    y = float(y)
    w = float(w)
    h = float(h)
    ws = _factorX(w)
    hs = _factorY(h)
    # If the rectangle is too small, then simply draw a pixel.
    if (ws <= 1.0) and (hs <= 1.0):
        _pixel(x, y)
    else:
        xs = _scaleX(x)
        ys = _scaleY(y)
        pygame.draw.rect(
            _surface,
            _pygameColor(_penColor),
            pygame.Rect(xs, ys-hs, ws, hs),
            int(round(_penRadius)))

def filledRectangle(x, y, w, h):
    """
    Draw on the background canvas a filled rectangle of width w and
    height h whose lower left point is (x, y).
    """
    global _surface
    _makeSureWindowCreated()
    x = float(x)
    y = float(y)
    w = float(w)
    h = float(h)
    ws = _factorX(w)
    hs = _factorY(h)
    # If the rectangle is too small, then simply draw a pixel.
    if (ws <= 1.0) and (hs <= 1.0):
        _pixel(x, y)
    else:
        xs = _scaleX(x)
        ys = _scaleY(y)
        pygame.draw.rect(
            _surface,
            _pygameColor(_penColor),
            pygame.Rect(xs, ys-hs, ws, hs),
            0)

def square(x, y, r):
    """
    Draw on the background canvas a square whose sides are of length
    2r, centered on (x, y).
    """
    _makeSureWindowCreated()
    rectangle(x-r, y-r, 2.0*r, 2.0*r)

def filledSquare(x, y, r):
    """
    Draw on the background canvas a filled square whose sides are of
    length 2r, centered on (x, y).
    """
    _makeSureWindowCreated()
    filledRectangle(x-r, y-r, 2.0*r, 2.0*r)

def polygon(x, y):
    """
    Draw on the background canvas a polygon with coordinates
    (x[i], y[i]).
    """
    global _surface
    _makeSureWindowCreated()
    # Scale X and Y values.
    xScaled = []
    for xi in x:
        xScaled.append(_scaleX(float(xi)))
    yScaled = []
    for yi in y:
        yScaled.append(_scaleY(float(yi)))
    points = []
    for i in range(len(x)):
        points.append((xScaled[i], yScaled[i]))
    points.append((xScaled[0], yScaled[0]))
    pygame.draw.polygon(
        _surface,
        _pygameColor(_penColor),
        points,
        int(round(_penRadius)))

def filledPolygon(x, y):
    """
    Draw on the background canvas a filled polygon with coordinates
    (x[i], y[i]).
    """
    global _surface
    _makeSureWindowCreated()
    # Scale X and Y values.
    xScaled = []
    for xi in x:
        xScaled.append(_scaleX(float(xi)))
    yScaled = []
    for yi in y:
        yScaled.append(_scaleY(float(yi)))
    points = []
    for i in range(len(x)):
        points.append((xScaled[i], yScaled[i]))
    points.append((xScaled[0], yScaled[0]))
    pygame.draw.polygon(_surface, _pygameColor(_penColor), points, 0)

def text(x, y, s):
    """
    Draw string s on the background canvas centered at (x, y).
    """
    _makeSureWindowCreated()
    x = float(x)
    y = float(y)
    xs = _scaleX(x)
    ys = _scaleY(y)
    font = pygame.font.SysFont(_fontFamily, _fontSize)
    text = font.render(s, 1, _pygameColor(_penColor))
    textpos = text.get_rect(center=(xs, ys))
    _surface.blit(text, textpos)

def picture(pic, x=None, y=None):
    """
    Draw pic on the background canvas centered at (x, y).  pic is an
    object of class picture.Picture. x and y default to the midpoint
    of the background canvas.
    """
    global _surface
    _makeSureWindowCreated()
    # By default, draw pic at the middle of the surface.
    if x is None:
        x = (_xmax + _xmin) / 2.0
    if y is None:
        y = (_ymax + _ymin) / 2.0
    x = float(x)
    y = float(y)
    xs = _scaleX(x)
    ys = _scaleY(y)
    ws = pic.width()
    hs = pic.height()
    picSurface = pic._surface # violates encapsulation
    _surface.blit(picSurface, [xs-ws/2.0, ys-hs/2.0, ws, hs])

def clear(c=WHITE):
    """
    Clear the background canvas to color c, where c is an
    object of class color.Color. c defaults to stddraw.WHITE.
    """
    _makeSureWindowCreated()
    _surface.fill(_pygameColor(c))

def save(f):
    """
    Save the window canvas to file f.
    """
    _makeSureWindowCreated()

    #if sys.hexversion >= 0x03000000:
    #    # Hack because Pygame without full image support
    #    # can handle only .bmp files.
    #    bmpFileName = f + '.bmp'
    #    pygame.image.save(_surface, bmpFileName)
    #    os.system('convert ' + bmpFileName + ' ' + f)
    #    os.system('rm ' + bmpFileName)
    #else:
    #    pygame.image.save(_surface, f)

    pygame.image.save(_surface, f)

#-----------------------------------------------------------------------

def _show():
    """
    Copy the background canvas to the window canvas.
    """
    _background.blit(_surface, (0, 0))
    pygame.display.flip()
    _checkForEvents()

def _showAndWaitForever():
    """
    Copy the background canvas to the window canvas. Then wait
    forever, that is, until the user closes the stddraw window.
    """
    _makeSureWindowCreated()
    _show()
    QUANTUM = .1
    while True:
        time.sleep(QUANTUM)
        _checkForEvents()

def show(msec=float('inf')):
    """
    Copy the background canvas to the window canvas, and
    then wait for msec milliseconds. msec defaults to infinity.
    """
    if msec == float('inf'):
        _showAndWaitForever()

    _makeSureWindowCreated()
    _show()
    _checkForEvents()

    # Sleep for the required time, but check for events every
    # QUANTUM seconds.
    QUANTUM = .1
    sec = msec / 1000.0
    if sec < QUANTUM:
        time.sleep(sec)
        return
    secondsWaited = 0.0
    while secondsWaited < sec:
        time.sleep(QUANTUM)
        secondsWaited += QUANTUM
        _checkForEvents()

#-----------------------------------------------------------------------

def _saveToFile():
    """
    Display a dialog box that asks the user for a file name.  Save the
    drawing to the specified file.  Display a confirmation dialog box
    if successful, and an error dialog box otherwise.  The dialog boxes
    are displayed using Tkinter, which (on some computers) is
    incompatible with Pygame. So the dialog boxes must be displayed
    from child processes.
    """
    import subprocess
    _makeSureWindowCreated()

    stddrawPath = os.path.realpath(__file__)

    childProcess = subprocess.Popen(
        [sys.executable, stddrawPath, 'getFileName'],
        stdout=subprocess.PIPE)
    so, se = childProcess.communicate()
    fileName = so.strip()

    if sys.hexversion >= 0x03000000:
        fileName = fileName.decode('utf-8')

    if fileName == '':
        return

    if not fileName.endswith(('.jpg', '.png')):
        childProcess = subprocess.Popen(
            [sys.executable, stddrawPath, 'reportFileSaveError',
             'File name must end with ".jpg" or ".png".'])
        return

    try:
        save(fileName)
        childProcess = subprocess.Popen(
            [sys.executable, stddrawPath, 'confirmFileSave'])
    except (pygame.error) as e:
        childProcess = subprocess.Popen(
            [sys.executable, stddrawPath, 'reportFileSaveError', str(e)])

def _checkForEvents():
    """
    Check if any new event has occured (such as a key typed or button
    pressed).  If a key has been typed, then put that key in a queue.
    """
    global _surface
    global _keysTyped

    #-------------------------------------------------------------------
    # Begin added by Alan J. Broder
    #-------------------------------------------------------------------
    global _mousePos
    global _mousePressed
    #-------------------------------------------------------------------
    # End added by Alan J. Broder
    #-------------------------------------------------------------------

    _makeSureWindowCreated()

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            sys.exit()
        elif event.type == pygame.KEYDOWN:
            _keysTyped = [event.unicode] + _keysTyped
        elif (event.type == pygame.MOUSEBUTTONUP) and \
             (event.button == 3):
            _saveToFile()

        #---------------------------------------------------------------
        # Begin added by Alan J. Broder
        #---------------------------------------------------------------
        # Every time the mouse button is pressed, remember
        # the mouse position as of that press.
        elif (event.type == pygame.MOUSEBUTTONDOWN) and \
             (event.button == 1): 
            _mousePressed = True
            _mousePos = event.pos                      
        #---------------------------------------------------------------
        # End added by Alan J. Broder
        #---------------------------------------------------------------

#-----------------------------------------------------------------------

# Functions for retrieving keys

def hasNextKeyTyped():
    """
    Return True if the queue of keys the user typed is not empty.
    Otherwise return False.
    """
    global _keysTyped
    return _keysTyped != []

def nextKeyTyped():
    """
    Remove the first key from the queue of keys that the the user typed,
    and return that key.
    """
    global _keysTyped
    return _keysTyped.pop()

#-----------------------------------------------------------------------
# Begin added by Alan J. Broder
#-----------------------------------------------------------------------

# Functions for dealing with mouse clicks 

def mousePressed():
    """
    Return True if the mouse has been left-clicked since the 
    last time mousePressed was called, and False otherwise.
    """
    global _mousePressed
    if _mousePressed:
        _mousePressed = False
        return True
    return False

def mouseX():
    """
    Return the x coordinate in user space of the location at
    which the mouse was most recently left-clicked. If a left-click
    hasn't happened yet, raise an exception, since mouseX() shouldn't
    be called until mousePressed() returns True.
    """
    global _mousePos
    if _mousePos:
        return _userX(_mousePos[0])      
    raise Exception(
        "Can't determine mouse position if a click hasn't happened")

def mouseY():
    """
    Return the y coordinate in user space of the location at
    which the mouse was most recently left-clicked. If a left-click
    hasn't happened yet, raise an exception, since mouseY() shouldn't
    be called until mousePressed() returns True.
    """
    global _mousePos
    if _mousePos:
        return _userY(_mousePos[1]) 
    raise Exception(
        "Can't determine mouse position if a click hasn't happened")

#-----------------------------------------------------------------------
# End added by Alan J. Broder
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------

# Initialize the x scale, the y scale, and the pen radius.

setXscale()
setYscale()
setPenRadius()
pygame.font.init()

#-----------------------------------------------------------------------

# Functions for displaying Tkinter dialog boxes in child processes.

def _getFileName():
    """
    Display a dialog box that asks the user for a file name.
    """
    root = Tkinter.Tk()
    root.withdraw()
    reply = tkFileDialog.asksaveasfilename(initialdir='.')
    sys.stdout.write(reply)
    sys.stdout.flush()
    sys.exit()

def _confirmFileSave():
    """
    Display a dialog box that confirms a file save operation.
    """
    root = Tkinter.Tk()
    root.withdraw()
    tkMessageBox.showinfo(title='File Save Confirmation',
                          message='The drawing was saved to the file.')
    sys.exit()

def _reportFileSaveError(msg):
    """
    Display a dialog box that reports a msg.  msg is a string which
    describes an error in a file save operation.
    """
    root = Tkinter.Tk()
    root.withdraw()
    tkMessageBox.showerror(title='File Save Error', message=msg)
    sys.exit()

#-----------------------------------------------------------------------

def _regressionTest():
    """
    Perform regression testing.
    """

    clear()

    setPenRadius(.5)
    setPenColor(ORANGE)
    point(0.5, 0.5)
    show(0.0)

    setPenRadius(.25)
    setPenColor(BLUE)
    point(0.5, 0.5)
    show(0.0)

    setPenRadius(.02)
    setPenColor(RED)
    point(0.25, 0.25)
    show(0.0)

    setPenRadius(.01)
    setPenColor(GREEN)
    point(0.25, 0.25)
    show(0.0)

    setPenRadius(0)
    setPenColor(BLACK)
    point(0.25, 0.25)
    show(0.0)

    setPenRadius(.1)
    setPenColor(RED)
    point(0.75, 0.75)
    show(0.0)

    setPenRadius(0)
    setPenColor(CYAN)
    for i in range(0, 100):
        point(i / 512.0, .5)
        point(.5, i / 512.0)
    show(0.0)

    setPenRadius(0)
    setPenColor(MAGENTA)
    line(.1, .1, .3, .3)
    line(.1, .2, .3, .2)
    line(.2, .1, .2, .3)
    show(0.0)

    setPenRadius(.05)
    setPenColor(MAGENTA)
    line(.7, .5, .8, .9)
    show(0.0)

    setPenRadius(.01)
    setPenColor(YELLOW)
    circle(.75, .25, .2)
    show(0.0)

    setPenRadius(.01)
    setPenColor(YELLOW)
    filledCircle(.75, .25, .1)
    show(0.0)

    setPenRadius(.01)
    setPenColor(PINK)
    rectangle(.25, .75, .1, .2)
    show(0.0)

    setPenRadius(.01)
    setPenColor(PINK)
    filledRectangle(.25, .75, .05, .1)
    show(0.0)

    setPenRadius(.01)
    setPenColor(DARK_RED)
    square(.5, .5, .1)
    show(0.0)

    setPenRadius(.01)
    setPenColor(DARK_RED)
    filledSquare(.5, .5, .05)
    show(0.0)

    setPenRadius(.01)
    setPenColor(DARK_BLUE)
    polygon([.4, .5, .6], [.7, .8, .7])
    show(0.0)

    setPenRadius(.01)
    setPenColor(DARK_GREEN)
    setFontSize(24)
    text(.2, .4, 'hello, world')
    show(0.0)

    #import picture as p
    #pic = p.Picture('saveIcon.png')
    #picture(pic, .5, .85)
    #show(0.0)

    # Test handling of mouse and keyboard events.
    setPenColor(BLACK)
    from itu.algs4.stdlib import stdio
    stdio.writeln('Left click with the mouse or type a key')
    while True:
        if mousePressed():
            filledCircle(mouseX(), mouseY(), .02)
        if hasNextKeyTyped():
            stdio.write(nextKeyTyped())
        show(0.0)

    # Never get here.
    show()

#-----------------------------------------------------------------------

def _main():
    """
    Dispatch to a function that does regression testing, or to a
    dialog-box-handling function.
    """
    import sys
    if len(sys.argv) == 1:
        _regressionTest()
    elif sys.argv[1] == 'getFileName':
        _getFileName()
    elif sys.argv[1] == 'confirmFileSave':
        _confirmFileSave()
    elif sys.argv[1] == 'reportFileSaveError':
        _reportFileSaveError(sys.argv[2])

if __name__ == '__main__':
    _main()



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
stdarray.py

The stdarray module defines functions related to creating, reading,
and writing one- and two-dimensional arrays.
"""

from itu.algs4.stdlib import stdio

#=======================================================================
# Array creation functions
#=======================================================================

def create1D(length, value=None):
    """
    Create and return a 1D array containing length elements, each
    initialized to value.
    """
    return [value] * length

#-----------------------------------------------------------------------

def create2D(rowCount, colCount, value=None):
    """
    Create and return a 2D array having rowCount rows and colCount
    columns, with each element initialized to value.
    """
    a = [None] * rowCount
    for row in range(rowCount):
        a[row] = [value] * colCount
    return a

#=======================================================================
# Array writing functions
#=======================================================================

def write1D(a):
    """
    Write array a to sys.stdout.  First write its length. bool objects
    are written as 0 and 1, not False and True.
    """
    length = len(a)
    stdio.writeln(length)
    for i in range(length):
        # stdio.writef('%9.5f ', a[i])
        element = a[i]
        if isinstance(element, bool):
            if element == True:
                stdio.write(1)
            else:
                stdio.write(0) 
        else:
            stdio.write(element)
        stdio.write(' ')
    stdio.writeln()

#-----------------------------------------------------------------------

def write2D(a):
    """
    Write two-dimensional array a to sys.stdout.  First write its
    dimensions. bool objects are written as 0 and 1, not False and True.
    """
    rowCount = len(a)
    colCount = len(a[0])
    stdio.writeln(str(rowCount) + ' ' + str(colCount))
    for row in range(rowCount):
        for col in range(colCount):
            #stdio.writef('%9.5f ', a[row][col])
            element = a[row][col]
            if isinstance(element, bool):
                if element == True:
                    stdio.write(1)
                else:
                    stdio.write(0)
            else:
                stdio.write(element)
            stdio.write(' ')
        stdio.writeln()

#=======================================================================
# Array reading functions
#=======================================================================

def readInt1D():
    """
    Read from sys.stdin and return an array of integers. An integer at
    the beginning of sys.stdin defines the array's length.
    """
    count = stdio.readInt()
    a = create1D(count, None)
    for i in range(count):
        a[i] = stdio.readInt()
    return a

#-----------------------------------------------------------------------

def readInt2D():
    """
    Read from sys.stdin and return a two-dimensional array of integers.
    Two integers at the beginning of sys.stdin define the array's
    dimensions.
    """
    rowCount = stdio.readInt()
    colCount = stdio.readInt()
    a = create2D(rowCount, colCount, 0)
    for row in range(rowCount):
        for col in range(colCount):
            a[row][col] = stdio.readInt()
    return a

#-----------------------------------------------------------------------

def readFloat1D():
    """
    Read from sys.stdin and return an array of floats. An integer at the
    beginning of sys.stdin defines the array's length.
    """
    count = stdio.readInt()
    a = create1D(count, None)
    for i in range(count):
        a[i] = stdio.readFloat()
    return a

#-----------------------------------------------------------------------

def readFloat2D():
    """
    Read from sys.stdin and return a two-dimensional array of floats.
    Two integers at the beginning of sys.stdin define the array's
    dimensions.
    """
    rowCount = stdio.readInt()
    colCount = stdio.readInt()
    a = create2D(rowCount, colCount, 0.0)
    for row in range(rowCount):
        for col in range(colCount):
            a[row][col] = stdio.readFloat()
    return a

#-----------------------------------------------------------------------

def readBool1D():
    """
    Read from sys.stdin and return an array of booleans. An integer at
    the beginning of sys.stdin defines the array's length.
    """
    count = stdio.readInt()
    a = create1D(count, None)
    for i in range(count):
        a[i] = stdio.readBool()
    return a

#-----------------------------------------------------------------------

def readBool2D():
    """
    Read from sys.stdin and return a two-dimensional array of booleans.
    Two integers at the beginning of sys.stdin define the array's
    dimensions.
    """
    rowCount = stdio.readInt()
    colCount = stdio.readInt()
    a = create2D(rowCount, colCount, False)
    for row in range(rowCount):
        for col in range(colCount):
            a[row][col] = stdio.readBool()
    return a

#=======================================================================

def _main():
    """
    For testing.
    """
    write2D(readFloat2D())
    write2D(readBool2D())

if __name__ == '__main__':
    _main()
